package eu.etaxonomy.taxeditor.prototype2.view;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.databinding.DataBindingContext;
import org.eclipse.core.databinding.beans.BeansObservables;
import org.eclipse.core.databinding.observable.set.ISetChangeListener;
import org.eclipse.core.databinding.observable.set.SetChangeEvent;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.text.source.SourceViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.DropTarget;
import org.eclipse.swt.dnd.DropTargetAdapter;
import org.eclipse.swt.dnd.DropTargetEvent;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.FocusListener;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.CoolBar;
import org.eclipse.swt.widgets.CoolItem;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Link;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.ToolBar;
import org.eclipse.swt.widgets.ToolItem;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.part.EditorPart;
import org.eclipse.ui.part.WorkbenchPart;

import com.swtdesigner.ResourceManager;
import com.swtdesigner.SWTResourceManager;

import eu.etaxonomy.cdm.model.name.BotanicalName;
import eu.etaxonomy.cdm.model.name.Rank;
import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.taxon.Synonym;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.prototype2.Activator;
import eu.etaxonomy.taxeditor.prototype2.controller.ActionDeleteTaxon;
import eu.etaxonomy.taxeditor.prototype2.controller.ActionOpenNameEditor;
import eu.etaxonomy.taxeditor.prototype2.controller.ActionOpenNewChildNameEditor;
import eu.etaxonomy.taxeditor.prototype2.controller.ActionSaveTaxon;
import eu.etaxonomy.taxeditor.prototype2.controller.TaxonTransfer;
import eu.etaxonomy.taxeditor.prototype2.model.PropertySheetNode;
import eu.etaxonomy.taxeditor.prototype2.view.nameviewersupport.GroupComposite;
import eu.etaxonomy.taxeditor.prototype2.view.nameviewersupport.NameComposite;
import eu.etaxonomy.taxeditor.prototype2.view.nameviewersupport.TesterooWritableSet;
import eu.etaxonomy.taxeditor.prototype2.view.propertysheetsupport.PropertySheetContentProvider;
import eu.etaxonomy.taxeditor.prototype2.view.propertysheetsupport.PropertySheetValueEditingSupport;
import eu.etaxonomy.taxeditor.prototype2.view.propertysheetsupport.PropertySheetValueLabelProvider;

/**
 * TODO: user should always know what kind of name - BotanicalName,
 *        ZoologicalName, etc. - he will be editing when a new taxon
 *        is created
 *        
 * @author p.ciardelli
 *
 */
public class NameEditorView extends EditorPart {
	/**
	 * The taxon the editor is editing 
	 */
	private Taxon taxon;
	/**
	 * The higher taxon of the taxon the editor is editing 
	 */
	private Taxon higherTaxon = null;	
	/**
	 * Shared listener that sets dirty state to true 
	 *  when any registered property changes 
	 */
	private PropertyChangeListener taxonChangeListener = new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent arg0) {
			dirty = true;
			firePropertyChange(PROP_DIRTY);
		}
	};	
	/**
	 * Arrays for the creation of synonyms - text and images 
	 */
	String [] relTypes = {"a synonym (type unknown)",
			"a homotypic synonym",
			"a heterotypic synoynm",
			"a basionym",
			"a replaced synonym",
			"a homonym",
			"an orthographic variant",
			"a misapplied name",
			"a concept relation"};
	String [] relImgs = {"unknown_no_bg.gif",
			"homosyn_no_bg.gif",
			"heterosyn_no_bg.gif",
			"basionym_no_bg.gif",
			"repsyn_no_bg.gif",
			"homonym_no_bg.gif",
			"orthovariant_no_bg.gif",
			"misapplied_no_bg.gif",
			"concept_no_bg.gif"};
	String [] conceptTypes = {"congruent","included in","includes","overlaps","excludes","doubtful"};
	/**
	 * If true, show "save" prompt before closing editor 
	 */
	public boolean dirty = false;
	String clickText;
	
	private Composite nameComposite;
	private Composite contentComposite;
	
	private PropertySheetViewer propertySheetViewer;
	private Composite parent;
	
	public static final String ID = "eu.etaxonomy.taxeditor.prototype2.view.nameeditorview"; //$NON-NLS-1$

	public NameEditorView() {
		
	}
	/**
	 * Create contents of the editor part
	 * @param parent
	 */
	@Override
	public void createPartControl(Composite parent) {
	
		this.parent = parent;
		parent.setLayout(new GridLayout());
		parent.setRedraw(true);
				
		
		createHigherTaxon();
		

		final CoolBar coolBar = new CoolBar(parent, SWT.NONE);
		new CoolItem(coolBar, SWT.PUSH).setImage(ResourceManager.getPluginImage(Activator.getDefault(), "icons/save_edit.gif"));
		
		final ToolBar toolBar = new ToolBar(parent, SWT.NONE);
//		toolBar.setBackground(Display.getCurrent().getSystemColor(SWT.COLOR_GRAY));

		final ToolItem itemSave = new ToolItem(toolBar, SWT.PUSH);
		itemSave.setToolTipText("Save taxon");
		itemSave.setImage(ResourceManager.getPluginImage(Activator.getDefault(), "icons/save_edit.gif"));
		itemSave.addSelectionListener(new SelectionListener() {
			public void widgetDefaultSelected(SelectionEvent e) {
				dirty = false;
				new ActionSaveTaxon(taxon).run();
			}
			public void widgetSelected(SelectionEvent e) {
				dirty = false;
				new ActionSaveTaxon(taxon).run();
			}
		});

		final ToolItem itemMove = new ToolItem(toolBar, SWT.DROP_DOWN);
		itemMove.setToolTipText("Move taxon");
		
		final Menu menuMove = new Menu(toolBar);
		addDropDown(itemMove, menuMove);

		final MenuItem moveTaxonMenuItem = new MenuItem(menuMove, SWT.NONE);
		moveTaxonMenuItem.setText("Move taxon to another higher taxon");
		moveTaxonMenuItem.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(final SelectionEvent e) {
				new MoveDialogView(getSite().getShell());				
				MoveDialogView dialog = new MoveDialogView(getSite().getShell());
				dialog.open(taxon);				
			}
		});

		final MenuItem taxonToSynonymMenuItem = new MenuItem(menuMove, SWT.NONE);
		taxonToSynonymMenuItem.setSelection(true);
		taxonToSynonymMenuItem.setText("Turn taxon's accepted name into a synonym");		
		taxonToSynonymMenuItem.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(final SelectionEvent e) {
				new MoveDialogView(getSite().getShell());				
				MoveDialogView dialog = new MoveDialogView(getSite().getShell());
				dialog.open(taxon);				
			}
		});

		itemMove.setImage(ResourceManager.getPluginImage(Activator.getDefault(), "icons/correction_change.gif"));

		final ToolItem itemDelete = new ToolItem(toolBar, SWT.NONE);
		itemDelete.setToolTipText("Delete taxon");
		itemDelete.setImage(ResourceManager.getPluginImage(Activator.getDefault(), "icons/delete_edit.gif"));
		itemDelete.addSelectionListener(new SelectionListener() {

			public void widgetDefaultSelected(SelectionEvent e) {
				// TODO Auto-generated method stub
				
			}
			public void widgetSelected(SelectionEvent e) {
				new ActionDeleteTaxon(taxon).run();
			}
		});
				
		new ToolItem(toolBar, SWT.SEPARATOR);

		final ToolItem itemAddRelationship = new ToolItem(toolBar, SWT.DROP_DOWN);
		itemAddRelationship.setToolTipText("Add nom. or tax. relation to taxon");		
		itemAddRelationship.setImage(ResourceManager.getPluginImage(Activator.getDefault(), "icons/rel_no_bg.gif"));

		/*
		 * Create menu to add relations to the taxon
		 */
		final Menu menuAddRel = new Menu(toolBar);
		addDropDown(itemAddRelationship, menuAddRel);

		for (int i = 0; i < relTypes.length; i++) {
			final MenuItem menuItem;
			if (relTypes[i].contains("concept")) {
				menuItem = new MenuItem(menuAddRel, SWT.CASCADE);
				
				final Menu conceptMenu = new Menu(menuItem);
				menuItem.setMenu(conceptMenu);
				
				for (int j = 0; j < conceptTypes.length; j++) {
					final MenuItem conceptMenuItem = new MenuItem(conceptMenu, SWT.CHECK);
					conceptMenuItem.setText(conceptTypes[j]);
				}	
			} else {
				menuItem = new MenuItem(menuAddRel, SWT.NONE);				
			}
			
			menuItem.setImage(ResourceManager.getPluginImage(Activator.getDefault(), "icons/" + relImgs[i]));
			menuItem.setText("Add " + relTypes[i]);
		}
		
		new ToolItem(toolBar, SWT.SEPARATOR);

		final ToolItem itemAddTaxon = new ToolItem(toolBar, SWT.DROP_DOWN);
		itemAddTaxon.setImage(ResourceManager.getPluginImage(Activator.getDefault(), "icons/add_edit.gif"));
		itemAddTaxon.setToolTipText("Add new taxon");

		/*
		 *  drop-down to add new "included in" taxon
		 */		
		final Menu menuAdd = new Menu(toolBar);
		addDropDown(itemAddTaxon, menuAdd);

		if (higherTaxon != null) {
			final MenuItem addToHigherTaxonItem = new MenuItem(menuAdd, SWT.NONE);
			addToHigherTaxonItem.setText("Add new taxon to \"" + higherTaxon.getName().getTitleCache() + "\"");
			addToHigherTaxonItem.addSelectionListener(new SelectionListener() {
				public void widgetDefaultSelected(SelectionEvent e) {
					// TODO Auto-generated method stub
					
				}
				public void widgetSelected(SelectionEvent e) {
					new ActionOpenNewChildNameEditor(higherTaxon).run();
				}		
			});
		}
		final MenuItem addTaxonItem = new MenuItem(menuAdd, SWT.NONE);
		addTaxonItem.setText("Add new taxon to this taxon");
		addTaxonItem.addSelectionListener(new SelectionListener() {
			public void widgetDefaultSelected(SelectionEvent e) {
				// TODO Auto-generated method stub
				
			}
			public void widgetSelected(SelectionEvent e) {
				new ActionOpenNewChildNameEditor(taxon).run();
			}		
		});

		contentComposite = new Composite(parent, SWT.NONE);
		contentComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		contentComposite.setBackground(Display.getCurrent().getSystemColor(SWT.COLOR_WHITE));
		final GridLayout gridLayout_2 = new GridLayout();
		gridLayout_2.marginHeight = 0;
		gridLayout_2.marginWidth = 0;
		gridLayout_2.numColumns = 2;
		contentComposite.setLayout(gridLayout_2);
		contentComposite.setRedraw(true);

		
		nameComposite = new Composite(contentComposite, SWT.NONE);
		nameComposite.setBackground(Display.getCurrent().getSystemColor(SWT.COLOR_WHITE));
		nameComposite.setLayoutData(new GridData(SWT.FILL, SWT.TOP, true, true));
		GridLayout gridLayout = new GridLayout();
		gridLayout.verticalSpacing = 10;
		nameComposite.setLayout(gridLayout);		
		nameComposite.setRedraw(true);
		
		
//		clickText = "Click here to start entering accepted name or enter its individual fields in the property sheet to the right";
//		NameViewer_ acceptedNameViewer = new AcceptedNameViewer(taxon, bindingContext, nameComposite);
//		acceptedNameViewer.setBindingContext(bindingContext);
//		acceptedNameViewer.configure(new NameViewerConfig());
		
		GroupComposite homoGroupComposite = new GroupComposite(nameComposite);
		homoGroupComposite.setTaxon(taxon);
		homoGroupComposite.setNameEditorView(this);
		
		new NameComposite(homoGroupComposite, taxon).setNameEditorView(this);
		
		TesterooWritableSet.getInstance().addSetChangeListener
		(new ISetChangeListener() {
			public void handleSetChange(SetChangeEvent event) {
				for ( Object addition: event.diff.getAdditions()) {
					if (addition instanceof Synonym)
						createHeterotypicalGroup((Synonym) addition);
				}
				nameComposite.layout();
				contentComposite.layout();
			}
		});
		
		// Listen for names being dragged outside of existing homotypic groups -
		//  user wants to create a new group
		// Drop functionality
		Transfer[] types = new Transfer[] {TaxonTransfer.getInstance()};
		int operations = DND.DROP_MOVE | DND.DROP_COPY | DND.DROP_DEFAULT;
		DropTarget target = new DropTarget(nameComposite, operations);
		target.setTransfer(types);
		target.addDropListener(dropTargetAdapter);
		
		/*
		 *  construct PropertySheetViewer, with bindings to model layer
		 */
		propertySheetViewer = new PropertySheetViewer(contentComposite);
		
		propertySheetViewer.setContentProvider(
				new PropertySheetContentProvider());
		propertySheetViewer.getPropertyValueColumn().setLabelProvider(
				new PropertySheetValueLabelProvider(propertySheetViewer));
		propertySheetViewer.getPropertyValueColumn().setEditingSupport(
				new PropertySheetValueEditingSupport(propertySheetViewer));
		
		setPropertySheetNodes(taxon.getName());		
	}
	
	/**
	 * Display hyperlinked higher taxon, if there is one
	 */
	private void createHigherTaxon() {
		Composite higherTaxonComposite = new Composite(parent, SWT.NONE);
		higherTaxonComposite.setLayout(new FillLayout());
		(new Label(higherTaxonComposite, SWT.NONE)).setText("Higher taxon:  ");
		if (higherTaxon == null)
			(new Label(higherTaxonComposite, SWT.NONE)).setText("none");
		else {
			Link higherTaxonLink = new Link(higherTaxonComposite, SWT.NONE);
			higherTaxonLink.setText("<a href=\"#\">" + higherTaxon.getName().getTitleCache() + "</a>");
			higherTaxonLink.addMouseListener(new MouseListener() {
				public void mouseDoubleClick(MouseEvent e) {
					new ActionOpenNameEditor(higherTaxon).run();					
				}
				public void mouseDown(MouseEvent e) {
					new ActionOpenNameEditor(higherTaxon).run();
				}
				public void mouseUp(MouseEvent e) {
				}		
			});
		}		
	}
	public void setPropertySheetNodes(TaxonNameBase taxonname) {
		propertySheetViewer.setInput(getPropertySheetNodes(taxonname));
		propertySheetViewer.setExpandedState(fullTitleNode, true);
	}

	private SourceViewer createAcceptedName_() {
		
//		Method[] methods = BotanicalName.class.getMethods();
//		for (int i = 0; i < methods.length; i++) {
//			System.out.println("Annotations:");
//			for (int j = 0; i < methods[j].getAnnotations().length; j++)
//				System.out.println("\t" + methods[i].getAnnotations()[j]);
//			System.out.println("Type: " + methods[i].getReturnType().toString());
//			System.out.println("Name:" + methods[i].getName());
//			System.out.println();
//		}
		// use annotations to find persistent methods
		
		
//		PropertyDescriptor descriptor;
//		try {
//			descriptor = new PropertyDescriptor("genus", taxonname.getClass());
//			try {
//				System.out.println("Property desc. test in createAcceptedName: " 
//						+ descriptor.getReadMethod().invoke(taxonname, null));
//			} catch (IllegalArgumentException e) {
//				// TODO Auto-generated catch block
//				e.printStackTrace();
//			} catch (IllegalAccessException e) {
//				// TODO Auto-generated catch block
//				e.printStackTrace();
//			} catch (InvocationTargetException e) {
//				// TODO Auto-generated catch block
//				e.printStackTrace();
//			}
//		} catch (IntrospectionException e) {
//			// TODO Auto-generated catch block
//			e.printStackTrace();
//		} 
		
		final SourceViewer sourceViewer = new SourceViewer(nameComposite, null, 0); // createSourceViewer(taxon.getName());
		
		sourceViewer.getTextWidget().setLayoutData(
				new GridData(SWT.FILL, SWT.CENTER, true, false, 2, 1));
		sourceViewer.getTextWidget().setFont(
				SWTResourceManager.getFont("Georgia", 12, SWT.NONE));		
		
		/*
		 *  if this is a new name, show text prompt to start entering data
		 */
		if (taxon.getName().getNameCache() == null) {
			clickText = "Click here to start entering accepted name or enter individual fields in the property sheet to the right";
			
			sourceViewer.getTextWidget().setText(clickText);
			sourceViewer.getTextWidget().setFont(SWTResourceManager.getFont("Georgia", 12, SWT.ITALIC));
			sourceViewer.getTextWidget().setForeground(SWTResourceManager.getColor(192, 192, 192));
			sourceViewer.getTextWidget().addFocusListener(new FocusListener() {
				public void focusGained(FocusEvent e) {
					if (!dirty) {
						sourceViewer.getTextWidget().setText("");
						sourceViewer.getTextWidget().setFont(SWTResourceManager.getFont("Georgia", 12, SWT.NONE));
						sourceViewer.getTextWidget().setForeground(SWTResourceManager.getColor(0, 0, 0));
//						dirty = true;
//						firePropertyChange(PROP_DIRTY);
					}
				}
				public void focusLost(FocusEvent e) {				
					if (sourceViewer.getTextWidget().getText() == "") {
						sourceViewer.getTextWidget().setForeground(SWTResourceManager.getColor(192, 192, 192));
						sourceViewer.getTextWidget().setText(clickText);
						sourceViewer.getTextWidget().setFont(SWTResourceManager.getFont("Georgia", 12, SWT.ITALIC));
						dirty = false;
					} else {
						dirty = true;
					}
				}
			});		
		}
		
		return sourceViewer;
	}
	


	/**
	 * Creates the drop-down menu next to every synonym for changing relationship type
	 * 
	 * @param label
	 * @param name
	 */
//	private void createRelationMenu(Control control, BotanicalName name) {
	private void createRelationMenu(final Label label) {
		
		final Menu menu = new Menu(label);
		label.setMenu(menu);
		
		// context menu usually only shows up w/ right menu click -
		//  this listener lets the left click activate it as well
		label.addMouseListener(new MouseListener() {
			public void mouseDown(MouseEvent e) {
				menu.setVisible(true);
			}
			public void mouseDoubleClick(MouseEvent e) { }
			public void mouseUp(MouseEvent e) {	}
		});
		
		createRelationMenuItem(menu, "delete_edit.gif", "Delete synonym from this taxon");
		createRelationMenuItem(menu, "correction_change.gif", "Move synonym to another taxon");
		
		new MenuItem(menu, SWT.SEPARATOR);
		
		for (int i= 0; i < relTypes.length; i++) {
			final String relImg = relImgs[i];
			String relText = "Turn this synonym into " + relTypes[i];
			createRelationMenuItem(menu, relImg, relText).addSelectionListener(new SelectionListener() {
				public void widgetDefaultSelected(SelectionEvent e) {	
				}
				public void widgetSelected(SelectionEvent e) {
					label.setImage(ResourceManager.getPluginImage(Activator.getDefault(), "icons/" + relImg));	
				}
			});
		}
	}
	
	private MenuItem createRelationMenuItem(Menu menu, String img, String text) {
		final MenuItem menuItem = new MenuItem(menu, SWT.NONE);
		menuItem.setImage(ResourceManager.getPluginImage(Activator.getDefault(), "icons/" + img));
		menuItem.setText(text);
		return menuItem;
	}
		
	@Override
	public void setFocus() {
		// Set the focus
	}

	@Override
	public void doSave(IProgressMonitor monitor) {
		// Do the Save operation
	}

	@Override
	public void doSaveAs() {
		// Do the Save As operation
	}

	@Override
	public void init(IEditorSite site, IEditorInput input) throws PartInitException {
		
		if (!(input instanceof IEditorInput))
			throw new PartInitException(
					"Invalid Input: Must be IFileEditorInput");
		
		if (input.getAdapter(Taxon.class) != null) {
			taxon = (Taxon) input.getAdapter(Taxon.class);
		} else {
			
			taxon = null;
		}
		
		// Get parent taxon
		if (taxon.getTaxonomicParent() != null)
				higherTaxon = taxon.getTaxonomicParent();
		
		// Register listeners for any change in accepted name or set of relations
		taxon.getName().addPropertyChangeListener("titleCache", taxonChangeListener);
		taxon.addPropertyChangeListener("synonyms", taxonChangeListener);
		
		setSite(site);
		setInput(input);
		
	}

	PropertySheetNode fullTitleNode;
	/**
	 * @param taxonName
	 * @return
	 */
	private List<PropertySheetNode> getPropertySheetNodes(TaxonNameBase taxonName) {
				
		BotanicalName botName = (BotanicalName) taxonName;
				
		List<PropertySheetNode> propertySheetNodes = new ArrayList<PropertySheetNode>();
		fullTitleNode = new PropertySheetNode("Full Title", BeansObservables.observeValue(botName, "titleCache"), 
				null, PropertySheetNode.EDITABLE);
		propertySheetNodes.add(fullTitleNode);
				
		// Note: only top-level nodes are "add"-ed - otherwise, they are
		//  instantiated with their parent node
		if (botName.getRank() == null) botName.setRank(Rank.GENUS());
		new PropertySheetNode("Rank", botName.getRank().getLabel(), 
				fullTitleNode, PropertySheetNode.NOT_EDITABLE);		
		new PropertySheetNode("Genus", BeansObservables.observeValue(botName, "uninomial"), 
				fullTitleNode, PropertySheetNode.EDITABLE);
		new PropertySheetNode("Species Epithet", BeansObservables.observeValue(botName, "specificEpithet"), 
				fullTitleNode, PropertySheetNode.EDITABLE);
		
		String author = 
			botName.getCombinationAuthorTeam() == null ? "" : 
				botName.getCombinationAuthorTeam().getTitleCache();
		new PropertySheetNode("Author", author, 
				fullTitleNode, PropertySheetNode.NOT_EDITABLE);

		return propertySheetNodes;
	}
	
	@Override
	public boolean isDirty() {
		return dirty;
	}

	@Override
	public boolean isSaveAsAllowed() {
		return false;
	}
	
	private static void addDropDown(final ToolItem item, final Menu menu) {
		item.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				if (event.detail == SWT.ARROW) {
					Rectangle rect = item.getBounds();
					Point pt = new Point(rect.x, rect.y + rect.height);
					pt = item.getParent().toDisplay(pt);
					menu.setLocation(pt.x, pt.y);
					menu.setVisible(true);
				}
			}
		});
	}
	
	private void createHeterotypicalGroup(Synonym heterosyn) {

		GroupComposite heteroGroupComposite = new GroupComposite(nameComposite);
		heteroGroupComposite.setTaxon(taxon);
		heteroGroupComposite.setNameEditorView(this);
		
		NameComposite nameComposite = new NameComposite(heteroGroupComposite, heterosyn, taxon);
		nameComposite.setNameEditorView(this);
		nameComposite.setFocus();
	}
	
	/**
	 * Creates a new homotypic group when a name composite is dropped outside of all existing
	 *  homotypic groups 
	 */
	DropTargetAdapter dropTargetAdapter = new DropTargetAdapter() {
		
		public void drop(DropTargetEvent event) {
			
			Synonym synonym = (Synonym) event.data; 
			
			GroupComposite homotypicGroupComposite = new GroupComposite(nameComposite);
			homotypicGroupComposite.setTaxon(taxon);
			homotypicGroupComposite.setNameEditorView(NameEditorView.this);
			
			new NameComposite(homotypicGroupComposite, synonym, taxon).setFocus();
			
			NameEditorView.this.contentComposite.layout();
		}		
	};	
}