// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.description;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CCombo;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;

import eu.etaxonomy.cdm.api.service.ILocationService;
import eu.etaxonomy.cdm.model.description.Distribution;
import eu.etaxonomy.cdm.model.description.PresenceAbsenceTermBase;
import eu.etaxonomy.cdm.model.location.NamedArea;
import eu.etaxonomy.cdm.model.reference.ReferenceBase;
import eu.etaxonomy.taxeditor.controller.GlobalController;
import eu.etaxonomy.taxeditor.editor.reference.ReferenceSelectComposite;
import eu.etaxonomy.taxeditor.model.DescriptionUtil;
import eu.etaxonomy.taxeditor.model.IDistributionInfoProvider;

/**
 * @author p.ciardelli
 * @created 03.04.2009
 * @version 1.0
 */
public class DistributionDialog extends Dialog {
	private static final Logger logger = Logger
			.getLogger(DistributionDialog.class);
	
	private IDistributionInfoProvider provider;

	private String title;
	private String message;
	
	private static final String PRESENCE_ABSENCE_SEPARATOR = "----------------------------";
	/**
	 * Number of area menus to show
	 */
	private static final int NUM_AREA_MENUS = 4; 
	/**
	 * UI elements
	 */
	private CCombo vocabTypeCombo;
	private CCombo[] areaCombos = new CCombo[NUM_AREA_MENUS];
//	private CCombo areaCombo1;
//	private CCombo areaCombo2;
//	private CCombo areaCombo3;
//	private CCombo areaCombo4;
	private CCombo presenceAbsenceCombo;
	private ReferenceSelectComposite referenceComposite;
	/**
	 * Distribution fields
	 */
	private NamedArea area;
	private PresenceAbsenceTermBase<?> presenceAbsenceStatus;
	private Distribution distribution;
	private Object vocabType;
	private ReferenceBase reference;
	/**
	 * Lists to populate drop downs
	 */
	private List<Object> vocabTypes;
//	private List<NamedArea> areas1;
//	private List<NamedArea> areas2;
//	private List<NamedArea> areas3;
//	private List<NamedArea> areas4;
//	private List<List<NamedArea>> areas; // = new ArrayList<NamedArea>[4];  
	private List<PresenceAbsenceTermBase<?>> presenceAbsenceTerms;
	private List<List<NamedArea>> areas = new ArrayList<List<NamedArea>>();
	private NamedArea[] selectedAreas = new NamedArea[NUM_AREA_MENUS];
	private Map<NamedArea, List<NamedArea>> areaChildren;
	private Map<NamedArea, NamedArea> areaParents;
	/**
	 * Static fields to persist default values
	 */
	private static ReferenceBase lastReference;
	private static PresenceAbsenceTermBase<?> lastPresenceAbsence;
	private static Object lastVocabType;

	/**
	 * @param shell
	 * @param dialogTitle
	 * @param dialogMessage
	 */
	public DistributionDialog(Shell shell, String dialogTitle,
			String dialogMessage) {
		super(shell);
	
		title = dialogTitle;
		message = dialogMessage;
		
		provider = DescriptionUtil.getDistributionInfoProvider();
		
		recallPreviousSelections();
	}

	/**
	 * Set inputs to selections from last time dialog was used.
	 */
	private void recallPreviousSelections() {
		if (lastVocabType != null)  {
			vocabType = lastVocabType;
		}
//		if (lastArea1 != null) {
//			area1 = lastArea1;
//		}
//		if (lastArea2 != null) {
//			area2 = lastArea2;
//		}
//		if (lastArea3 != null) {
//			area3 = lastArea3;
//		}
//		if (lastArea4 != null) {
//			area4 = lastArea4;
//		}
		if (lastPresenceAbsence != null) {
			presenceAbsenceStatus = lastPresenceAbsence; 
		}
		if (lastReference != null) {
			reference = lastReference;
		}
	}

	/**
	 * @param dialogTitle 
	 * @param dialogMessage 
	 * @return
	 */
	public static Distribution getDistribution(String dialogTitle,
			String dialogMessage) {
		DistributionDialog dialog = new DistributionDialog(GlobalController.getShell(), 
				dialogTitle, 
				dialogMessage);
		if (dialog.open() == Window.CANCEL) {
			return null;
		}
		return dialog.getDistribution();
	}

	/**
	 * @return
	 */
	private Distribution getDistribution() {
		return distribution;
	}
	
    /*y
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.window.Window#configureShell(org.eclipse.swt.widgets.Shell)
     */
    protected void configureShell(Shell shell) {
        super.configureShell(shell);
        if (title != null) {
			shell.setText(title);
		}
    }
	
    protected Control createDialogArea(Composite parent) {
        // create composite
    	Composite composite = (Composite) super.createDialogArea(parent);
        // create message
        if (message != null) {
            createLabel(message, composite, parent);
        }
    	createLabel("Select an area vocabulary:", composite, parent);
        // create vocab type
        vocabTypeCombo = createCombo(composite);
        vocabTypeCombo.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				selectVocabType();
			}
		});
        // create areas
        createLabel("Select an area:", composite, parent);
        for (int i = 0; i < NUM_AREA_MENUS; i++) {
        	areaCombos[i] = createCombo(composite); 
            areaCombos[i].addSelectionListener(new AreaComboListener(i));
        }
        // create presence / absence
        createLabel("Select presence / absence:", composite, parent);
		presenceAbsenceCombo =  new CCombo(composite, SWT.BORDER);
		presenceAbsenceCombo.setLayoutData(new GridData(GridData.GRAB_HORIZONTAL
                | GridData.HORIZONTAL_ALIGN_FILL));
		presenceAbsenceCombo.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				selectPresenceAbsenceTerm();
			}
		});
		// create reference
		referenceComposite = new ReferenceSelectComposite(composite);

        populateVocabTypes();
		populateAreas();
		populatePresenceAbsenceTerms();
		populateReference();
        
        applyDialogFont(composite);
        return composite;
    }

    class AreaComboListener extends SelectionAdapter {
    	private int i;
		AreaComboListener(int i) {
    		this.i = i;
    	}
		@Override
		public void widgetSelected(SelectionEvent e) {
			selectArea(i);
		}
    }
    
	/**
	 * @return
	 */
	private CCombo createCombo(Composite composite) {
		CCombo combo = new CCombo(composite, SWT.BORDER);
		combo.setLayoutData(new GridData(GridData.GRAB_HORIZONTAL
				| GridData.HORIZONTAL_ALIGN_FILL));
		return combo;
	}
    
	/**
	 * 
	 */
	private void populateVocabTypes() {
		int index = 0;
		int i = 0;
		for (Object vocabType : getVocabTypes()) {
			if (vocabType == null) {
				vocabTypeCombo.add("");
			} else {
				if (vocabType.equals(this.vocabType)) {
					index = i;
				}
				vocabTypeCombo.add(provider.getVocabularyTypeLabel(vocabType));
			}
			i++;
		}
		vocabTypeCombo.select(index);
	}
	
	/**
	 * 
	 */
	private void populateAreas() {
		clearAreas();
		if (vocabType != null) {
			int index = 0;
			int i = 0;
			for (NamedArea area : getNamedAreas(vocabType)) {
				if (area == null) {
					areaCombos[0].add("");
				} else {
					if (area.equals(this.area)) {
						index = i;
					}
					areaCombos[0].add(area.getDescription());
				}
				i++;
			}
			areaCombos[0].select(index);
		}
		for (int j = 1; j < NUM_AREA_MENUS; j++) {
			NamedArea selectedArea = selectedAreas[j - 1];
			if (selectedArea != null) {
				int index = 0;
				int i = 0;
				for (NamedArea area : getNamedAreas(selectedArea, j)) {
					if (area == null) {
						areaCombos[i].add("");
					} else {
						if (area.equals(this.area)) {
							index = i;
						}
						areaCombos[0].add(area.getDescription());
					}
					i++;
				}
				areaCombos[0].select(index);
			}
		}
//		areaCombo2.setVisible(false);
//		areaCombo3.setVisible(false);
//		areaCombo4.setVisible(false);
//		areaCombo2.moveBelow(referenceComposite);
//		areaCombo3.moveBelow(referenceComposite);
//		areaCombo4.moveBelow(referenceComposite);
	}

	
	/**
	 * @param namedArea
	 * @return
	 */
	private List<NamedArea> getNamedAreas(NamedArea namedArea, int depth) {
		List<NamedArea> areasTemp = new ArrayList<NamedArea>();
		areasTemp.add(null);
		areasTemp.addAll(provider.getVocabularyNextLevelAreas(namedArea));
		return areasTemp;
	}

	/**
	 * @param vocabType
	 * @return
	 */
	private List<NamedArea> getNamedAreas(Object vocabType) {
		List<NamedArea> areasTemp = new ArrayList<NamedArea>();
		areasTemp.add(null);
		areasTemp.addAll(provider.getVocabularyTopLevelAreas(vocabType));
//		areas
		return areasTemp;
	}

	/**
	 * 
	 */
	private void populatePresenceAbsenceTerms() {
		int index = 0;
		int i = 0;
		for (PresenceAbsenceTermBase<?> term : getPresenceAbsenceTerms()) {
			if (term == null) {
				if (i == 0) {
					presenceAbsenceCombo.add("");
				} else {
					presenceAbsenceCombo.add(PRESENCE_ABSENCE_SEPARATOR);
				}
			} else {
				presenceAbsenceCombo.add(provider.getPresenceAbsenceTermLabel(term));				
				if (this.presenceAbsenceStatus != null && this.presenceAbsenceStatus.equals(term)) {
					index = i;
				}
			}
			i++;
		}
		presenceAbsenceCombo.select(index);
	}

	/**
	 * 
	 */
	private void populateReference() {
		if (reference != null) {
			referenceComposite.setReference(reference);
		}
	}
	
	/**
	 * @return
	 */
	private List<Object> getVocabTypes() {
		if (vocabTypes == null) {
			vocabTypes = new ArrayList<Object>();
			// Add empty entry for selection menu
			vocabTypes.add(null);
			vocabTypes.addAll(provider.getVocabularyTypes());
		}
		return vocabTypes;
	}
	
	/**
	 * @return
	 */
	private List<PresenceAbsenceTermBase<?>> getPresenceAbsenceTerms() {
		if (presenceAbsenceTerms == null) {
			presenceAbsenceTerms = new ArrayList<PresenceAbsenceTermBase<?>>();
			// Add empty entry for selection menu
			presenceAbsenceTerms.add(null);
			
			for (PresenceAbsenceTermBase<?> term : provider.getPresenceTerms()) {
				presenceAbsenceTerms.add(term);
			}
			presenceAbsenceTerms.add(null);
			for (PresenceAbsenceTermBase<?> term : provider.getAbsenceTerms()) {
				presenceAbsenceTerms.add(term);
			}
		}
		return presenceAbsenceTerms;
	}

	/**
	 * @param i
	 */
	protected void selectArea(int depth) {
		// This selection is made currently selected area
		int i = areaCombos[depth].getSelectionIndex();
		selectedAreas[depth] = getSelectedArea(depth, i);
		
		// Combos below this area combo are cleared
//		clearArea(depth+1);
		populateAreas();
				
		// Next combo repopulated with this combo's named area children, if any
//		populateArea(i+1);
		
		// Contents of this combo become current area list
		
//		switch (i) {
//		case 1:
//			areaCombo1.removeAll();
//		case 2:
//			areaCombo2.removeAll();
//		case 3:
//			areaCombo3.removeAll();
//		case 4:
//			areaCombo4.removeAll();	
//		}
	}

	/**
	 * @param i
	 * @param index
	 * @return
	 */
	private NamedArea getSelectedArea(int depth, int index) {
		return areas.get(depth).get(index);
	}

	/**
	 * 
	 */
	protected void selectVocabType() {
		int i = vocabTypeCombo.getSelectionIndex();
		selectVocabType(i);
	}
	
	private void selectVocabType(int i) {
		vocabType = getVocabTypes().get(i);
		
		clearAreas();
		populateAreas();
//		populateArea(1);
	}
	
	/**
	 * 
	 */
	protected void selectPresenceAbsenceTerm() {
		int i = presenceAbsenceCombo.getSelectionIndex();
		presenceAbsenceStatus = presenceAbsenceTerms.get(i);
	}

	/**
	 * @param string
	 * @param parent 
	 * @param composite 
	 * @return 
	 */
	private Label createLabel(String string, Composite composite, Composite parent) {
		Label label = new Label(composite, SWT.WRAP);
        label.setText(string);
        GridData data = new GridData(GridData.GRAB_HORIZONTAL
                | GridData.GRAB_VERTICAL | GridData.HORIZONTAL_ALIGN_FILL
                | GridData.VERTICAL_ALIGN_CENTER);
        data.widthHint = convertHorizontalDLUsToPixels(IDialogConstants.MINIMUM_MESSAGE_AREA_WIDTH);
        label.setLayoutData(data);
        label.setFont(parent.getFont());
        return label;
	}

	/**
	 * @param i
	 */
	private void clearArea(int i) {
		for (int j = i; j < NUM_AREA_MENUS; j++) {
			areaCombos[j].removeAll();
		}
	}

	/**
	 * 
	 */
	private void clearAreas() {
		clearArea(0);
	}
	
	protected void createButtonsForButtonBar(Composite parent) {
		createButton(parent, IDialogConstants.OK_ID,
                IDialogConstants.OK_LABEL, true);
        createButton(parent, IDialogConstants.CANCEL_ID,
                IDialogConstants.CANCEL_LABEL, false);
    }
	
	protected void buttonPressed(int buttonId) {
		if (IDialogConstants.OK_ID == buttonId) {
			okPressed();
		} else if (IDialogConstants.CANCEL_ID == buttonId) {
			cancelPressed();
		}
	}

	protected void okPressed() {
		createDistribution();
		setStaticFields();
		
		super.okPressed();
	}
	
	/**
	 * 
	 */
	private void setStaticFields() {
		lastPresenceAbsence = getPresenceAbsenceTerm();
		lastReference = getReference(); 
		lastVocabType = getVocabType();
	}

	/**
	 * @return
	 */
	private Object getVocabType() {
		int i = vocabTypeCombo.getSelectionIndex();
		return vocabTypes.get(i);
	}

	/**
	 * @return
	 */
	private ReferenceBase getReference() {
		return referenceComposite.getReference();
	}

	/**
	 * @return
	 */
	private PresenceAbsenceTermBase<?> getPresenceAbsenceTerm() {
		int i = presenceAbsenceCombo.getSelectionIndex();
		return presenceAbsenceTerms.get(i);
	}
	
	/**
	 * 
	 */
	private void createDistribution() {
		distribution = Distribution.NewInstance(area, presenceAbsenceStatus);
	}
}