/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.preference;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;
import org.eclipse.jface.preference.PreferencePage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;

import eu.etaxonomy.cdm.model.description.Feature;
import eu.etaxonomy.taxeditor.controller.PreferencesController;
import eu.etaxonomy.taxeditor.model.CdmSessionDataRepository;

/**
 * @author p.ciardelli
 * @created 17.09.2008
 * @version 1.0
 */
public class FeaturePreferences extends PreferencePage implements
		IWorkbenchPreferencePage {
	private static final Logger logger = Logger
			.getLogger(FeaturePreferences.class);

	public static final String PLUGIN_ID = "eu.etaxonomy.taxeditor.preferences.feature";
	
	private Map<Feature, Button> featureButtons;

	private Composite container;

	public FeaturePreferences() {
		super();
		setDescription("Choose which features you would like to use for descriptive elements.");
	}


	/**
	 * Create contents of the preference page
	 * @param parent
	 */
	@Override
	public Control createContents(Composite parent) {

		featureButtons = new HashMap<Feature, Button>();
		
		container = new Composite(parent, SWT.NULL);
		final GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 3;
		container.setLayout(gridLayout);

		List<Feature> features = 
				CdmSessionDataRepository.getDefault().getFeatures();
		
		// Purpose of this code block: 
		//	fill first column, then second, then third
		Object[] array = features.toArray();
		int count = features.size();
		int colheight = count / 3;
		if (count % 3 > 0) {
			colheight++;
		}
		for (int i = 0; i < colheight; i++) {
			createButton((Feature) array[i]);
			if (i + colheight < count) {
				createButton((Feature) array[i + colheight]);
			}
			if (i + (colheight * 2) < count) {
				createButton((Feature) array[i + (colheight * 2)]);
			}
		}
		return container;
	}


	/**
	 * @param object
	 */
	private void createButton(Feature feature) {
		Button button = new Button(container, SWT.CHECK);
		button.setText(feature.getLabel());
		featureButtons.put(feature, button);
		
		if (PreferencesController.getFeaturePreference(feature)) {
			button.setSelection(true);
		}
	}


	/* (non-Javadoc)
	 * @see org.eclipse.ui.IWorkbenchPreferencePage#init(org.eclipse.ui.IWorkbench)
	 */
	public void init(IWorkbench workbench) {
		setPreferenceStore(PreferencesController.getPrefStore());
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.preference.PreferencePage#performDefaults()
	 */
	protected void performDefaults() {
		
		// Set all buttons to show feature
		for (Button button : featureButtons.values()) {
			button.setSelection(true);
		}
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.preference.PreferencePage#performOk()
	 */
	public boolean performOk() {
		
		for (Feature feature : featureButtons.keySet()) {
			Button button = featureButtons.get(feature);
			PreferencesController.setFeaturePreference(feature, button.getSelection());
		}
		
		return true;
	}
}
