package eu.etaxonomy.taxeditor.propertysheet;

import java.util.Collection;
import java.util.Vector;

import org.apache.log4j.Logger;
import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.IPropertySource;
import org.eclipse.ui.views.properties.PropertyDescriptor;

/**
 * @author p.ciardelli
 * @created 08.05.2008
 * @version 1.0
 */
abstract public class CollectionPropertySource implements IPropertySource {
	private static final Logger logger = Logger
			.getLogger(CollectionPropertySource.class);

	protected Vector<PropertyDescriptor> descriptors = new Vector<PropertyDescriptor>();
	protected Collection collection;
	private Object collectionOwner;
	
	public CollectionPropertySource(Collection collection) {
		this.collection = collection;
		for (Object item : collection) {
			addDescriptor(item);
		}
	}

	public CollectionPropertySource(Collection collection, Object collectionOwner) {
		this.collection = collection;
		this.collectionOwner = collectionOwner;
		for (Object item : collection) {
			addDescriptor(item);
		}
	}
	
	protected Object getCollectionOwner() {
		return collectionOwner;
	}
	
	protected void addDescriptor(Object item) {
		String itemDisplayName = getItemDisplayName(item);
		descriptors.addElement(
				new PropertyDescriptor(item, itemDisplayName));
	}
	
	/**
	 * Returns the label for the name of the property sheet entry.
	 *  Return "" if the entry should have no label.
	 * 
	 * @param item
	 * @return
	 */
	abstract protected String getItemDisplayName(Object item);

	public Object getEditableValue() {
		return this;
	}
	
	public IPropertyDescriptor[] getPropertyDescriptors() {
		return (IPropertyDescriptor[]) descriptors.toArray(
                new IPropertyDescriptor[descriptors.size()]);
	}

	abstract public Object getPropertyValue(Object id);

	public boolean isPropertySet(Object id) {
		return false;
	}	
	
	public void resetPropertyValue(Object id) {}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.views.properties.IPropertySource#setPropertyValue(java.lang.Object, java.lang.Object)
	 */
	abstract public void setPropertyValue(Object id, Object value);
	
	/**
	 * Returns the string that should appear in the parent
	 *  entry's value field, which is usually "".
	 * 
	 * @return String
	 * @see java.lang.Object#toString()
	 */
	@Override
	abstract public String toString();
}
