/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.propertysheet.name;

import org.apache.log4j.Logger;

import eu.etaxonomy.cdm.model.name.ZoologicalName;

/**
 * Property source for ZoologicalName entries.
 *  
 * @author p.ciardelli
 * @created 21.05.2008
 * @version 1.0
 */
public class ZoologicalNamePropertySource extends NonViralNamePropertySource {
	private static final Logger logger = Logger
			.getLogger(ZoologicalNamePropertySource.class);

	private ZoologicalName zoologicalName;
	
    // Property unique keys
	public static final String P_ID_PUBLICATIONYEAR = "publicationyear";
    // Property display keys
	public static final String P_PUBLICATIONYEAR = "099:Publication Year";
		
	public ZoologicalNamePropertySource(ZoologicalName name) {
		super(name);
		this.zoologicalName = name;
		addDescriptor(P_ID_PUBLICATIONYEAR);
	}

	@Override
	public Object getPropertyValue(Object id) {
        if (id.equals(P_ID_NOMENCLATURAL_CODE)) {
        	return "ICZN";
        }
        if (id.equals(P_ID_PUBLICATIONYEAR)) {
        	String publicationYear = Integer.toString(zoologicalName.getPublicationYear());
        	if (publicationYear.equals("0")) {
        		return "";
        	} else {
        		return publicationYear;
        	}
        }
		return super.getPropertyValue(id);
	}
}
