/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.propertysheet.reference;

import org.apache.log4j.Logger;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.swt.graphics.Color;
import org.eclipse.ui.views.properties.PropertyDescriptor;

import eu.etaxonomy.cdm.model.reference.Book;
import eu.etaxonomy.cdm.model.reference.Journal;
import eu.etaxonomy.cdm.model.reference.ReferenceBase;
import eu.etaxonomy.taxeditor.ITaxEditorConstants;
import eu.etaxonomy.taxeditor.TaxEditorPlugin;
import eu.etaxonomy.taxeditor.controller.GlobalController;

/**
 * This subclass is for cases where only one subclass of <code>ReferenceBase</code>
 * can be edited in the property source. 
 * <p>
 * For instance, if the user is editing an
 * <code>Article</code>, one of its fields would be a 
 * <code>SingleRefTypePropertySource</code> restricted to a <code>Journal</code>,
 * and the reference type drop-down would be replaced by the non-editable 
 * value "Journal".
 * 
 * @author p.ciardelli
 * @created 19.11.2008
 * @version 1.0
 */
public class SingleRefTypePropertySource extends ReferencePropertySource {
	private static final Logger logger = Logger
			.getLogger(SingleRefTypePropertySource.class);
	
	public static final String BOOK = "Book";
	public static final String JOURNAL = "Journal";

	private String referenceType;
	
	public SingleRefTypePropertySource(ReferenceBase reference, String referenceType) {
		super(reference);
		
		this.referenceType = referenceType;
	}

	protected PropertyDescriptor createReferenceTypeDescriptor() {
		return new PropertyDescriptor(P_ID_REFERENCETYPE, P_REFERENCETYPE) {
			public ILabelProvider getLabelProvider() {
				return super.getLabelProvider();
			}
		};
	}
	
	public Object getPropertyValue(Object id) {
		
		// Reference Type: show non-editable reference type in property value
		if (id.equals(P_ID_REFERENCETYPE)) {			
			return referenceType;
		}
		
		return super.getPropertyValue(id);
	}
}
