/**
* Copyright (C) 2009 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.propertysheet.type.wizard;

import java.util.Set;

import org.apache.log4j.Logger;
import org.eclipse.core.databinding.observable.list.WritableList;
import org.eclipse.jface.wizard.IWizardPage;
import org.eclipse.jface.wizard.Wizard;

import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.model.name.NameTypeDesignation;
import eu.etaxonomy.cdm.model.name.SpecimenTypeDesignation;
import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.name.TypeDesignationBase;
import eu.etaxonomy.cdm.model.occurrence.Specimen;
import eu.etaxonomy.taxeditor.model.CdmUtil;

/**
 * @author p.ciardelli
 * @created 11.02.2009
 * @version 1.0
 */
public class TypeListWizard extends Wizard {
	private static Logger logger = Logger.getLogger(TypeListWizard.class);

	private TaxonNameBase name;
	private WritableList typeDesignationsList;
	
	/**
	 * @param name
	 */
	public TypeListWizard(TaxonNameBase name) {
		super();
		
		this.name = name;
		this.typeDesignationsList = new WritableList();
		if (name != null) {
			if (CdmUtil.isNameSupraSpecific(name)) {
				this.typeDesignationsList.addAll(name.getNameTypeDesignations());				
			} else {
				this.typeDesignationsList.addAll(name.getSpecimenTypeDesignations());
			}
		}
	}

	public void addPages(){
		IWizardPage chooseTypePage = new ListTypeWizardPage(name, typeDesignationsList);
    	addPage(chooseTypePage);
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.wizard.Wizard#performFinish()
	 */
	@Override
	public boolean performFinish() {
		
		if (name != null) {
			
			// Remove name's designations
			Set existingDesignations; 
			if (CdmUtil.isNameSupraSpecific(name)) {
				existingDesignations = name.getNameTypeDesignations();
			} else {
				existingDesignations = name.getSpecimenTypeDesignations();				
			}
			
			for (Object typeDesignation : existingDesignations) {	
				if (typeDesignation instanceof TypeDesignationBase) {
					name.removeTypeDesignation((TypeDesignationBase) typeDesignation);
				}
			}
			
			// Then re-add them from list
			for (Object typeDesignation : typeDesignationsList) {
				if (typeDesignation instanceof SpecimenTypeDesignation) {
					
					Specimen specimen = CdmBase.deproxy(((SpecimenTypeDesignation) typeDesignation).getTypeSpecimen(), Specimen.class);
					
					name.addSpecimenTypeDesignation(							
							specimen,
							((SpecimenTypeDesignation) typeDesignation).getTypeStatus(),
							((SpecimenTypeDesignation) typeDesignation).getCitation(),
							((SpecimenTypeDesignation) typeDesignation).getCitationMicroReference(),
							((SpecimenTypeDesignation) typeDesignation).getOriginalNameString(),
							((SpecimenTypeDesignation) typeDesignation).isNotDesignated(),
							false);
				}
				
				if (typeDesignation instanceof NameTypeDesignation) {
					name.addNameTypeDesignation(
							((NameTypeDesignation) typeDesignation).getTypeName(), 
							((NameTypeDesignation) typeDesignation).getCitation(), 
							((NameTypeDesignation) typeDesignation).getCitationMicroReference(),
							((NameTypeDesignation) typeDesignation).getOriginalNameString(), 
							((NameTypeDesignation) typeDesignation).isRejectedType(), 
							((NameTypeDesignation) typeDesignation).isConservedType(), 
							((NameTypeDesignation) typeDesignation).isLectoType(), 
							((NameTypeDesignation) typeDesignation).isNotDesignated(), 
							false);
				}
			}
		}
		return true;
	}
}
