/**
 * 
 */
package eu.etaxonomy.taxeditor.editor.descriptiontree.handler;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.common.NotDefinedException;
import org.eclipse.core.commands.operations.IUndoableOperation;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.TreePath;
import org.eclipse.jface.viewers.TreeSelection;
import org.eclipse.swt.widgets.Event;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.forms.editor.FormEditor;
import org.eclipse.ui.handlers.HandlerUtil;

import eu.etaxonomy.cdm.model.description.Feature;
import eu.etaxonomy.cdm.model.description.TaxonDescription;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.editor.TaxonEditorInput;
import eu.etaxonomy.taxeditor.operations.CreateDescriptionElementOperation;
import eu.etaxonomy.taxeditor.operations.DeleteTaxonDescriptionOperation;
import eu.etaxonomy.taxeditor.operations.IPostOperationEnabled;

/**
 * @author p.ciardelli
 *
 */
public class DeleteDescriptionHandler extends AbstractHandler {
	private static final Logger logger = Logger
			.getLogger(DeleteDescriptionHandler.class);

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.IHandler#execute(org.eclipse.core.commands.ExecutionEvent)
	 */
	public Object execute(ExecutionEvent event) throws ExecutionException {
		
		IEditorPart editor = HandlerUtil.getActiveEditor(event);
		if (editor instanceof FormEditor) {
			editor = ((FormEditor) editor).getActiveEditor();
		}
		IEditorInput input = editor.getEditorInput();
		if (input instanceof TaxonEditorInput) {
			Taxon taxon = ((TaxonEditorInput) input).getTaxon();
			
			ISelection selection = HandlerUtil.getCurrentSelection(event);
			if (selection instanceof TreeSelection) {
				TreePath[] paths = ((TreeSelection) selection).getPaths();
				TaxonDescription description = (TaxonDescription) (paths[0]).getFirstSegment();
							
				IUndoableOperation operation;
				try {
					// TODO use undo context specific to editor
					operation = new DeleteTaxonDescriptionOperation(event.getCommand().getName(), 
							EditorUtil.getUndoContext(), taxon, description, (IPostOperationEnabled) editor);
					EditorUtil.executeOperation(operation);
				} catch (NotDefinedException e) {
					logger.warn("Command name not set");
				}
			}
		} 
		return null;
	}

}
