// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.name.handler;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IUndoableOperation;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.handlers.HandlerUtil;

import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.model.taxon.ITreeNode;
import eu.etaxonomy.cdm.model.taxon.Synonym;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonNode;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.editor.TaxonEditorInput;
import eu.etaxonomy.taxeditor.operations.ChangeSynonymToAcceptedTaxonOperation;
import eu.etaxonomy.taxeditor.operations.IPostOperationEnabled;
import eu.etaxonomy.taxeditor.propertysheet.name.SynonymPropertySource;

/**
 * @author n.hoffmann
 * @created 21.04.2009
 * @version 1.0
 */
public class ChangeSynonymToAcceptedTaxonHandler extends AbstractHandler implements IPostOperationEnabled {
	private static final Logger logger = Logger
			.getLogger(ChangeSynonymToAcceptedTaxonHandler.class);
	
	private IEditorPart editor;

	private Taxon taxon;

	private ITreeNode parentNode;

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.IHandler#execute(org.eclipse.core.commands.ExecutionEvent)
	 */
	public Object execute(ExecutionEvent event) throws ExecutionException {
		editor = HandlerUtil.getActiveEditor(event);
		Shell shell = HandlerUtil.getActiveShell(event);
		IEditorInput input = editor.getEditorInput();
		
		if (!(input instanceof TaxonEditorInput)) {
			logger.error("Editor input is not TaxonEditorInput");
			return null;
		}

		// Get synonym from selection
		StructuredSelection selection = (StructuredSelection) HandlerUtil.getActiveMenuSelection(event);
		if (!(selection.getFirstElement() instanceof SynonymPropertySource)) {
			logger.error("Expected selection containing SynonymPropertySource");
			return null;
		}

		Synonym synonym = (Synonym) ((SynonymPropertySource) selection.getFirstElement()).getTaxonBase();
		
		// Force user to save taxon - not really necessary though, is it?
		if (!EditorUtil.forceUserSave(editor, shell)) {
			return null;
		}

		// Get taxon
		taxon = ((TaxonEditorInput) input).getTaxon();
		parentNode = ((TaxonEditorInput) input).getTaxonNode().getParent();
						
		// Create new taxon with no parent			
		IUndoableOperation operation = new ChangeSynonymToAcceptedTaxonOperation("Change synonym to accepted taxon", EditorUtil.getUndoContext(), 
				taxon, parentNode, synonym, this); //$NON-NLS-1$
		EditorUtil.executeOperation(operation);
		
		return null;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.operations.IPostOperationEnabled#postOperation(eu.etaxonomy.cdm.model.common.CdmBase)
	 */
	public boolean postOperation(CdmBase objectAffectedByOperation) {
		
		// Redraw existing editor
		((IPostOperationEnabled) editor).postOperation(null);
		
		editor.doSave(null);
		
		if (objectAffectedByOperation instanceof TaxonNode) {
		
			// Open new unsaved editor with existing taxon's parent as temporary parent
			TaxonNode newNode = (TaxonNode) objectAffectedByOperation;
//			TaxonNode newNode = parentNode.addChild(newTaxon);
					
			try {
				// TODO
				/*
				 * This doesn't work b/c newNode hasn't been committed yet, and therefore
				 *  CdmStore.getTaxonService().getTaxonNodeByUuid(taxonNodeUuid);
				 *  doesn't work yet.
				 */
				IEditorPart newEditor = EditorUtil.open(newNode.getUuid());
				
			} catch (PartInitException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		return true;
	}
}
