/**
 * 
 */
package eu.etaxonomy.taxeditor.forms.entitysections;

import java.util.Set;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.widgets.Composite;

import eu.etaxonomy.cdm.model.common.AnnotatableEntity;
import eu.etaxonomy.cdm.model.common.Annotation;
import eu.etaxonomy.cdm.model.common.AnnotationType;
import eu.etaxonomy.cdm.model.common.Language;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * 
 * @author n.hoffmann
 * @created Nov 5, 2009
 * @version 1.0
 */
public class AnnotationSection extends AbstractEntitySetSection<AnnotatableEntity>{

	/**
	 * @param parent
	 * @param style
	 */
	public AnnotationSection(Composite parent, AnnotatableEntity annotatableEntity, IPropertyChangeListener listener, int style) {
		super(parent, annotatableEntity, "Annotations", listener, style);
	}
	

	/*
	 * (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.forms.AbstractEntitySetSection#createDynamicContents()
	 */
	@Override
	public void createDynamicContents() {
		
		Set<Annotation> annotations = null;
		if(getEntity() != null){
			annotations = getEntity().getAnnotations();
		}
		
		if(annotations == null || annotations.size() == 0){
			toolkit.createLabel(container, "No annotations yet.");
		}else{
			for(Annotation annotation : annotations){
				new AnnotationComposite(container, annotation, new RemoveListener(annotation), this, SWT.NULL);
			}
		}
	}
	
	protected class RemoveListener extends SelectionAdapter{
		private Annotation annotation;
		
		public RemoveListener(Annotation annotation){
			this.annotation = annotation;
		}
		
		/* (non-Javadoc)
		 * @see org.eclipse.swt.events.SelectionAdapter#widgetSelected(org.eclipse.swt.events.SelectionEvent)
		 */
		@Override
		public void widgetSelected(SelectionEvent e) {
			getEntity().removeAnnotation(annotation);
			internalUpdateSection(true);
		}
	}
	
	

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.forms.AbstractEntitySetSection#getAddAction()
	 */
	@Override
	protected IAction getAddAction() {
		Action addAnnotation = new Action("add", Action.AS_PUSH_BUTTON){
			/* (non-Javadoc)
			 * @see org.eclipse.jface.action.Action#run()
			 */
			@Override
			public void run() {
				String text = "";
				AnnotationType type = AnnotationType.EDITORIAL();
				Language language = CdmStore.getDefaultLanguage();
				
				Annotation annotation = Annotation.NewInstance(text, type, language);
				getEntity().addAnnotation(annotation);
				if(! getSection().isExpanded())
					getSection().setExpanded(true);
				internalUpdateSection(true);
			}
		};
		addAnnotation.setToolTipText("Create a new annotation");
		return addAnnotation;
	}	
}
