/**
 * 
 */
package eu.etaxonomy.taxeditor.forms.entitysections;

import java.util.List;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.widgets.Composite;

import eu.etaxonomy.cdm.model.common.Credit;
import eu.etaxonomy.cdm.model.common.IdentifiableEntity;

/**
 * 
 * @author n.hoffmann
 * @created Nov 16, 2009
 * @version 1.0
 */
public class CreditSection extends AbstractEntitySetSection<IdentifiableEntity> {

	/**
	 * @param parent
	 * @param entity
	 * @param style
	 */
	public CreditSection(Composite parent, IdentifiableEntity entity, IPropertyChangeListener listener,
			int style) {
		super(parent, entity, "Credits", listener, style);
	}


	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.forms.AbstractEntitySetSection#createDynamicContents()
	 */
	@Override
	protected void createDynamicContents() {
		List<Credit> credits = null;
		if(getEntity() != null){
			credits = getEntity().getCredits();
		}
		
		if(credits == null || credits.size() == 0){
			toolkit.createLabel(container, "No credits yet.");
		}else{
			for(Credit credit : credits){
				new CreditComposite(container, credit, new RemoveListener(credit), this, SWT.NULL);
			}
		}
	}
	
	private class RemoveListener extends SelectionAdapter{
		private Credit credit;
		
		public RemoveListener(Credit credit){
			this.credit = credit;
		}
		
		/* (non-Javadoc)
		 * @see org.eclipse.swt.events.SelectionAdapter#widgetSelected(org.eclipse.swt.events.SelectionEvent)
		 */
		@Override
		public void widgetSelected(SelectionEvent e) {
			getEntity().removeCredit(credit);
			internalUpdateSection(true);
		}
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.forms.AbstractEntitySetSection#getAddAction()
	 */
	@Override
	protected IAction getAddAction() {
		Action addCredit = new Action("add", Action.AS_PUSH_BUTTON){
			/* (non-Javadoc)
			 * @see org.eclipse.jface.action.Action#run()
			 */
			@Override
			public void run() {
				Credit credit = Credit.NewInstance(null, null);
				getEntity().addCredit(credit);
				internalUpdateSection(true);
				if(! getSection().isExpanded())
					getSection().setExpanded(true);
				internalUpdateSection(true);
			}
		};
		addCredit.setToolTipText("Create a new credit entry");
		return addCredit;
	}

}
