// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.forms.entitysections;

import org.apache.log4j.Logger;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.widgets.Composite;

import eu.etaxonomy.cdm.model.common.DescriptionElementSource;
import eu.etaxonomy.taxeditor.forms.TextWithLabelComposite;
import eu.etaxonomy.taxeditor.forms.selectioncomposites.NameSelectionComposite;
import eu.etaxonomy.taxeditor.forms.selectioncomposites.ReferenceSelectionComposite;

/**
 * @author n.hoffmann
 * @created Nov 16, 2009
 * @version 1.0
 * @param <T>
 */
public class DescriptionElementSourceComposite extends AbstractEntitySetElementComposite<DescriptionElementSource> {
	/**
	 * @param parent
	 * @param element
	 * @param removeListener
	 * @param propertyChangelistener
	 * @param style
	 */
	public DescriptionElementSourceComposite(Composite parent,
			DescriptionElementSource element, SelectionListener removeListener,
			IPropertyChangeListener propertyChangelistener, int style) {
		super(parent, element, removeListener, propertyChangelistener, style);
	}

	private static final Logger logger = Logger
			.getLogger(DescriptionElementSourceComposite.class);
	

	private NameSelectionComposite nameSelection;


	private ReferenceSelectionComposite citationSelection;


	private TextWithLabelComposite microCitation;
	

	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.forms.entitysections.OriginalSourceComposite#createControls(org.eclipse.swt.widgets.Composite, int)
	 */
	@Override
	public void createControls(Composite parent, int style) {
		citationSelection = new ReferenceSelectionComposite(parent, "Citation", null, this, SWT.NULL);		
		microCitation = new TextWithLabelComposite(parent, "Citation Detail", null, this, SWT.NULL);
		nameSelection = new NameSelectionComposite(parent, "Name in Source", null, this, SWT.NULL);
	}
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.forms.entitysections.OriginalSourceComposite#setElement(eu.etaxonomy.cdm.model.common.OriginalSourceBase)
	 */
	@Override
	public void setElement(DescriptionElementSource element) {
		this.element = element;
		citationSelection.setSelection(element.getCitation());
		microCitation.setText(element.getCitationMicroReference());
		nameSelection.setSelection(element.getNameUsedInSource());
	}
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.forms.entitysections.OriginalSourceComposite#propertyChange(org.eclipse.jface.util.PropertyChangeEvent)
	 */
	@Override
	public void propertyChange(PropertyChangeEvent event) {
		super.propertyChange(event);
		element.setCitation(citationSelection.getSelection());
		element.setCitationMicroReference(microCitation.getText());
		element.setNameUsedInSource(nameSelection.getSelection());
	}
}
