/**
 * 
 */
package eu.etaxonomy.taxeditor.forms.entitysections;

import java.util.Set;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;

import eu.etaxonomy.cdm.model.common.Extension;
import eu.etaxonomy.cdm.model.common.IdentifiableEntity;

/**
 * @author nho
 *
 */
public class ExtensionSection extends AbstractEntitySetSection<IdentifiableEntity> {

	/**
	 * @param parent
	 * @param entity
	 * @param style
	 */
	public ExtensionSection(Composite parent, IdentifiableEntity entity,
			IPropertyChangeListener listener, int style) {
		super(parent, entity, "Extensions", listener, style);
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.forms.AbstractEntitySetSection#createDynamicContents()
	 */
	@Override
	protected void createDynamicContents() {
		Set<Extension> extensions = null;
		if(getEntity() != null){
			extensions = getEntity().getExtensions();
		}
		
		if(extensions == null || extensions.size() == 0){
			toolkit.createLabel(container, "No extensions yet.");
		}else{
			for(Extension extension : extensions){
				new ExtensionComposite(container, extension, new RemoveListener(extension), this, SWT.NULL);
			}
		}
	}
	
	private class RemoveListener extends SelectionAdapter{
		private Extension extension;
		
		public RemoveListener(Extension extension){
			this.extension = extension;
		}
		
		/* (non-Javadoc)
		 * @see org.eclipse.swt.events.SelectionAdapter#widgetSelected(org.eclipse.swt.events.SelectionEvent)
		 */
		@Override
		public void widgetSelected(SelectionEvent e) {
			getEntity().removeExtension(extension);
			internalUpdateSection(true);
		}
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.forms.AbstractEntitySetSection#getAddAction()
	 */
	@Override
	protected IAction getAddAction() {
		Action addExtension = new Action("add", Action.AS_PUSH_BUTTON){
			/* (non-Javadoc)
			 * @see org.eclipse.jface.action.Action#run()
			 */
			@Override
			public void run() {
				Extension extension = Extension.NewInstance();
				getEntity().addExtension(extension);
				internalUpdateSection(true);
				if(! getSection().isExpanded())
					getSection().setExpanded(true);
				internalUpdateSection(true);
			}
		};
		addExtension.setToolTipText("Create a new extension");
		return addExtension;
	}

}
