/**
 * 
 */
package eu.etaxonomy.taxeditor.forms.selectioncomposites;

import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

import eu.etaxonomy.cdm.model.common.IdentifiableEntity;
import eu.etaxonomy.taxeditor.forms.AbstractFormComposite;

/**
 * 
 * @author n.hoffmann
 * @created Nov 17, 2009
 * @version 1.0
 * @param <T>
 */
public abstract class AbstractSelectionComposite<T extends IdentifiableEntity> extends AbstractFormComposite implements SelectionListener{

	protected T selection;
	
	protected Label label;
	protected Text text;
	protected Button button;

	/**
	 * @param parent
	 * @param style
	 */
	public AbstractSelectionComposite(Composite parent, String labelString, T selection, IPropertyChangeListener listener, int style) {
		super(parent, listener, style);
		
		this.selection = selection;
		
		this.setLayoutData(new GridData(SWT.FILL, SWT.TOP, true, false));
		GridLayout layout = (GridLayout) this.getLayout();
		layout.numColumns = 2;
				
		if(labelString != null){
			label = toolkit.createLabel(this, labelString, SWT.WRAP);
			GridData labelData = new GridData();
			labelData.horizontalSpan = 2;
			label.setLayoutData(labelData);
		}
		
		text = toolkit.createText(this, "", SWT.BORDER | SWT.WRAP);
		text.setEditable(false);
		text.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
		
		button = toolkit.createButton(this, "Browse", SWT.PUSH);
		button.addSelectionListener(this);
		
		updateText();
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.swt.events.SelectionListener#widgetDefaultSelected(org.eclipse.swt.events.SelectionEvent)
	 */
	public void widgetDefaultSelected(SelectionEvent e) {
		// do nothing
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.swt.events.SelectionListener#widgetSelected(org.eclipse.swt.events.SelectionEvent)
	 */
	public void widgetSelected(SelectionEvent e) {
		firePropertyChangeEvent(null);
	}
	
	/**
	 * Return the selected object
	 * 
	 * @return
	 */
	public T getSelection(){
		return selection;
	}
	
	/**
	 * 
	 * @param selection
	 */
	public void setSelection(T selection){
		this.selection = selection;
		updateText();
	}

	protected void updateText(){
		if(selection != null)
			text.setText(selection.getTitleCache());
	}
}
