/**
 * 
 */
package eu.etaxonomy.taxeditor.propertysheet.authors;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.apache.log4j.Logger;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.dialogs.TitleAreaDialog;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;

import eu.etaxonomy.cdm.model.agent.AgentBase;
import eu.etaxonomy.cdm.model.agent.Person;
import eu.etaxonomy.cdm.model.agent.Team;
import eu.etaxonomy.taxeditor.dialogs.filteredSelection.FilteredAgentSelectionDialog;
import eu.etaxonomy.taxeditor.dialogs.filteredSelection.FilteredPersonSelectionDialog;

/**
 * @author p.ciardelli
 *
 */
public class TeamAuthorsListDialog extends TitleAreaDialog {
	private static final Logger logger = Logger
			.getLogger(TeamAuthorsListDialog.class);

	/**
	 * @param shell 
	 * @param team
	 * @return
	 */
	public static List<Person> getAuthorsList(Shell shell, Team team) {
		TeamAuthorsListDialog dialog = new TeamAuthorsListDialog(shell, team);
		if (dialog.open() == Window.CANCEL) {
			return null;
		}
		return dialog.getAuthorsList();
	}

	/**
	 * @return
	 */
	private List<Person> getAuthorsList() {
		return authors;
	}

	private Table authorListTable;
	private Team team;
	private List<Person> authors = new ArrayList<Person>();
	private TableViewer authorListViewer;
	private Button btnRemove;

	private static final int UP = -1;
	private static final int DOWN = 1;
	
	/**
	 * @param parentShell
	 * @param team 
	 */
	protected TeamAuthorsListDialog(Shell parentShell, Team team) {
		super(parentShell);
		
		this.team = team;
		this.authors.addAll(team.getTeamMembers());
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.TitleAreaDialog#createContents(org.eclipse.swt.widgets.Composite)
	 */
	@Override
	protected Control createContents(Composite parent) {
	    Control contents = super.createContents(parent);
				
		setTitle("Edit authors in team");
		setMessage("Add or remove the members of this team, and move their position in the list.");

		this.getShell().setText("Authors");
		
		return contents;
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.Dialog#createDialogArea(org.eclipse.swt.widgets.Composite)
	 */
	@Override
	protected Control createDialogArea(Composite parent) {
		Composite container = (Composite) super.createDialogArea(parent);
		
        // create composite
        Composite composite = new Composite(container, SWT.NULL);
        composite.setLayoutData(new GridData(GridData.FILL_BOTH));
		
		final GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 4;
		composite.setLayout(gridLayout);

		Button btnAdd = new Button(composite, SWT.NONE);
		btnAdd.setText("Add ...");
		btnAdd.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				addAuthor();
			}
		});

		btnRemove = new Button(composite, SWT.NONE);
		btnRemove.setText("Remove");
		btnRemove.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, true, false));
		btnRemove.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				removeSelectedAuthor();
			}
		});
		setRemoveEnabled(false);

		final Button btnUp = new Button(composite, SWT.NONE);
		btnUp.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false));
		btnUp.setText("Up");
		btnUp.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				moveAuthor(UP);
			}
		});

		final Button btnDown = new Button(composite, SWT.NONE);
		btnDown.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false));
		btnDown.setText("Down");
		btnDown.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				moveAuthor(DOWN);
			}
		});
		
		authorListViewer = new TableViewer(composite, SWT.BORDER | SWT.SINGLE);
		createColumns(authorListViewer);
		authorListViewer.setContentProvider(new AuthorListContentProvider());
		authorListViewer.setLabelProvider(new AuthorListLabelProvider());
		authorListViewer.addSelectionChangedListener(new ISelectionChangedListener () {
			public void selectionChanged(SelectionChangedEvent event) {
				setRemoveEnabled(true);
			}
		});
		
		// TODO extend table viewer to show Person fields
		authorListTable = authorListViewer.getTable();
		authorListTable.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true,
				true, 4, 1));
		
		populateAuthors();
		
		return container;
	}

	private void createColumns(TableViewer viewer) {
		Table table = viewer.getTable();
		String[] titles = {"Cache", "Nomenclatural Cache", "First Name", "Last Name"};
		int[] bounds = { 130, 130, 90, 90};

		for (int i = 0; i < titles.length; i++) {
			TableViewerColumn column = new TableViewerColumn(viewer, SWT.NONE);
			column.getColumn().setText(titles[i]);
			column.getColumn().setWidth(bounds[i]);
			column.getColumn().setResizable(true);
			column.getColumn().setMoveable(true);
		}
		table.setHeaderVisible(true);
		table.setLinesVisible(true);
	}
	
	/**
	 * @param direction
	 */
	private void moveAuthor(int direction) {
		Person author = getSelectedAuthor();
		if (author == null) {
			return;
		}
		int index = authors.indexOf(author);
		if (direction == UP && index > 0) {
			Collections.swap(authors, index, index - 1);
		}
		if (direction == DOWN && index < authors.size() - 1) {
			Collections.swap(authors, index, index + 1);
		}
		authorListViewer.refresh();
	}

	/**
	 * @param b
	 */
	private void setRemoveEnabled(boolean enabled) {
		btnRemove.setEnabled(enabled);
	}

	/**
	 * 
	 */
	private void populateAuthors() {
		authorListViewer.setInput(authors);
	}

	/**
	 * 
	 */
	private void removeSelectedAuthor() {
		authors.remove(getSelectedAuthor());
		authorListViewer.refresh();
	}

	/**
	 * @return
	 */
	private Person getSelectedAuthor() {
		if (authorListViewer.getSelection() == null) {
			return null;
		}
		return (Person) ((IStructuredSelection) authorListViewer.getSelection()).getFirstElement();
	}

	/**
	 * 
	 */
	private void addAuthor() {
		Person person = FilteredPersonSelectionDialog.selectPerson(getParentShell(), null);
		if (person != null) {
			authors.add(person);
			authorListViewer.refresh();
		}
//		AgentBase agent = FilteredAgentSelectionDialog.selectAgent(getParentShell(), null);
//		if (agent == null) {
//			return;
//		}
//		if (agent instanceof Person) {
//			authors.add((Person) agent);
//			authorListViewer.refresh();			
//		} else {
//			MessageDialog.openError(getParentShell(), "Not a person", "'" + agent.getTitleCache() + "' is not a person.");
//		}
	}
	
    /* (non-Javadoc)
     * @see org.eclipse.jface.dialogs.Dialog#okPressed()
     */
    @Override
    protected void okPressed() {
    	// Remove all members from team
    	List<Person> removeAuthors = new ArrayList<Person>();
    	removeAuthors.addAll(team.getTeamMembers());
    	team.getTeamMembers().removeAll(removeAuthors);
    	
    	// Add all members
    	for (Person person : authors) {
    		team.addTeamMember(person);
    	}
    	team.setTitleCache(team.generateTitle());
    	super.okPressed();
    }
	
	class AuthorListContentProvider implements IStructuredContentProvider {

		/* (non-Javadoc)
		 * @see org.eclipse.jface.viewers.IStructuredContentProvider#getElements(java.lang.Object)
		 */
		public Object[] getElements(Object inputElement) {
			return ((List<?>) inputElement).toArray();
		}

		/* (non-Javadoc)
		 * @see org.eclipse.jface.viewers.IContentProvider#dispose()
		 */
		public void dispose() {}

		/* (non-Javadoc)
		 * @see org.eclipse.jface.viewers.IContentProvider#inputChanged(org.eclipse.jface.viewers.Viewer, java.lang.Object, java.lang.Object)
		 */
		public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {}		
	}
	
	class AuthorListLabelProvider extends LabelProvider implements ITableLabelProvider{

		/* (non-Javadoc)
		 * @see org.eclipse.jface.viewers.ITableLabelProvider#getColumnImage(java.lang.Object, int)
		 */
		public Image getColumnImage(Object element, int columnIndex) {
			return null;
		}

		/* (non-Javadoc)
		 * @see org.eclipse.jface.viewers.ITableLabelProvider#getColumnText(java.lang.Object, int)
		 */
		public String getColumnText(Object element, int columnIndex) {
			Person person = (Person) element;
			switch (columnIndex) {
			case 0:
				return person.getTitleCache();
			case 1:
				return person.getNomenclaturalTitle();
			case 2:
				return person.getFirstname();
			case 3:
				return person.getLastname();
			}
			return null;
		}
		
	}
}
