/**
* Copyright (C) 2009 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.propertysheet.type.wizard;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import org.apache.log4j.Logger;
import org.eclipse.core.databinding.observable.list.WritableList;
import org.eclipse.core.runtime.Assert;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;

import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.model.name.NameTypeDesignation;
import eu.etaxonomy.cdm.model.name.NameTypeDesignationStatus;
import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.name.TypeDesignationBase;
import eu.etaxonomy.cdm.model.reference.ReferenceBase;
import eu.etaxonomy.taxeditor.editor.name.NameSelectComposite;
import eu.etaxonomy.taxeditor.editor.reference.ReferenceSelectComposite;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.store.VocabularyStore;

/**
 * @author p.ciardelli
 * @created 09.03.2009
 * @version 1.0
 */
public class ChooseNameTypeWizardPage extends WizardPage {
	private static Logger logger = Logger
			.getLogger(ChooseNameTypeWizardPage.class);

	private NameTypeDesignation typeDesignation;
	private WritableList typeDesignationsList;
	
	private Button btnRejected;
	private Button btnConserved;
	private Button btnNotDesignated;

	private ReferenceSelectComposite referenceComposite;
	private NameSelectComposite nameComposite;

	private Group statusGroup;
	private Combo statusCombo;

	private NameTypeDesignationStatus[] typeStatusArray;

	/**
	 * @param typeDesignation
	 * @param name
	 * @param typeDesignationsList 
	 */
	public ChooseNameTypeWizardPage(TypeDesignationBase<?> typeDesignation,
			TaxonNameBase<?, ?> name, WritableList typeDesignationsList) {
		super("");
		
		Assert.isTrue(typeDesignation == null || typeDesignation instanceof NameTypeDesignation);
		
		this.typeDesignation = (NameTypeDesignation) typeDesignation;
		this.typeDesignationsList = typeDesignationsList;
		
		setTitle("Create or edit type designation");
		setDescription("Create or edit type designation for '" + name.getTitleCache() + "\".");
		logger.trace(this.getClass().getSimpleName() + " created.");
	}
	
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.IDialogPage#createControl(org.eclipse.swt.widgets.Composite)
	 */
	public void createControl(Composite parent) {
		
		// Create composite for popup dialog	
		Composite container = new Composite(parent, SWT.NULL);
		container.setLayout(new GridLayout());
		setControl(container);
			
		// Create name input composite
		nameComposite = new NameSelectComposite(container);
				
		statusGroup = new Group(container, SWT.NONE);
		statusGroup.setLayout(new GridLayout());
		statusGroup.setLayoutData(new GridData(SWT.FILL, SWT.TOP, true, false));
		statusGroup.setText("Designation type status");
		
		// Create designation type status dropdown 
		statusCombo = new Combo(statusGroup, SWT.BORDER | SWT.READ_ONLY);
		statusCombo.setLayoutData(new GridData(SWT.FILL, SWT.TOP, true, false));
		typeStatusArray = new NameTypeDesignationStatus[PreferencesUtil.getPreferredSpecimenTypeDesignationStatus().size()];
		int i = 0;
		int selectedIndex = -1;
		
		NameTypeDesignationStatus typeDesignationStatus = null;
		if (typeDesignation != null && typeDesignation.getTypeStatus() != null) {
			typeDesignationStatus = CdmBase.deproxy(typeDesignation.getTypeStatus(), 
					NameTypeDesignationStatus.class);
		}
					
		for (Object typeStatusObject : VocabularyStore.getNameTypeDesignationStatus()) {
			NameTypeDesignationStatus typeStatus = CdmBase.deproxy(typeStatusObject, 
					NameTypeDesignationStatus.class);
			String label = typeStatus.getLabel();
			typeStatusArray[i] = typeStatus;
			if (typeDesignationStatus != null) {
				if (typeStatus.equals(typeDesignationStatus)) {
					selectedIndex = i;
				}
			}			
			i++;
			statusCombo.add(label);
		}
		
		// Set menu to type designation status if exists
		statusCombo.select(selectedIndex);
		statusCombo.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				updatePage();
			}
		});
		
		// Create 2-columned composite for boolean values
		Composite booleansComposite = new Composite(statusGroup, SWT.NULL);
		booleansComposite.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 2;
		gridLayout.marginHeight = 5;
		gridLayout.marginWidth = 0;
		booleansComposite.setLayout(gridLayout);
		
		btnRejected = new Button(booleansComposite, SWT.CHECK);
		btnRejected.setText("Is rejected type");
		btnRejected.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
		
		btnConserved = new Button(booleansComposite, SWT.CHECK);
		btnConserved.setText("Is conserved type");
		btnConserved.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
				
		// Create composite w checkbox for "not designated"
		Composite notDesignatedComposite = new Composite(container, SWT.NONE);
		GridData gridData = new GridData(SWT.FILL, SWT.TOP, true, false);
		gridData.heightHint = 40;
		notDesignatedComposite.setLayoutData(gridData);
		GridLayout gridLayout2 = new GridLayout();
		gridLayout2.marginHeight = 0;
		notDesignatedComposite.setLayout(gridLayout2);
		
		btnNotDesignated = new Button(notDesignatedComposite, SWT.CHECK);
		btnNotDesignated.setText("Type not designated");
		btnNotDesignated.setLayoutData(new GridData(SWT.FILL, SWT.BEGINNING, true, false));

		// Set any boolean values
		if (typeDesignation != null) {
			if (typeDesignation.isRejectedType()) {
				btnRejected.setSelection(true);
			}
			if (typeDesignation.isConservedType()) {
				btnConserved.setSelection(true);
			}
			setIsNotDesignated(typeDesignation.isNotDesignated());
		}
		
		// Create reference input composite
		referenceComposite = new ReferenceSelectComposite(container);
		
		// Set text to reference text if exists
		if (typeDesignation != null && typeDesignation.getCitation() != null) {
			referenceComposite.setMicroReference(typeDesignation.getCitationMicroReference());
			if (typeDesignation.getCitation() != null) {
				referenceComposite.setReference(typeDesignation.getCitation());
			}
		}
		
		// Set properties
		if (typeDesignation != null) {
			nameComposite.setName(typeDesignation.getTypeName());
		}
		
		// Add listeners
		nameComposite.addPropertyChangeListener(new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent evt) {
				updatePage();
			}
		});

		btnNotDesignated.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				updatePage();
			}			
		});
	}
	
	/**
	 * @param notDesignated
	 */
	private void setIsNotDesignated(boolean notDesignated) {
		statusGroup.setEnabled(!notDesignated);
		statusCombo.setEnabled(!notDesignated);
		btnConserved.setEnabled(!notDesignated);
		btnRejected.setEnabled(!notDesignated);
		btnNotDesignated.setSelection(notDesignated);
	}

	@Override
	public boolean canFlipToNextPage() {
		return isPageComplete();
	}

	public boolean isPageComplete() {
		return nameComposite.getName() != null;
	}
	
	private void updatePage() {
		setIsNotDesignated(btnNotDesignated.getSelection());
		getWizard().getContainer().updateButtons();
	}
	
    public void setPageComplete(boolean complete) {
        super.setPageComplete(complete);
        
        if (complete) {
        	
        	TaxonNameBase<?, ?> typeName = nameComposite.getName();

        	NameTypeDesignationStatus status = null;
        	boolean isRejectedType = false;
        	boolean isConservedType = false;
        	boolean isNotDesignated = btnNotDesignated.getSelection();
        	if (!isNotDesignated) {
            	status = typeStatusArray[statusCombo.getSelectionIndex()];
        		isRejectedType = btnRejected.getSelection();
        		isConservedType = btnConserved.getSelection();
        	}
        	
        	ReferenceBase<?> citation = referenceComposite.getReference();
        	String citationMicroReference = referenceComposite.getMicroReference();
        	    
        	if (typeDesignation == null) {
        		typeDesignation = new TemporaryNameTypeDesignation(typeName, citation, citationMicroReference,
        				null, status, isRejectedType, isConservedType, isNotDesignated);           		
        	} else {
        		typeDesignation.setTypeStatus(status);
        		typeDesignation.setNotDesignated(isNotDesignated);
        		typeDesignation.setRejectedType(isRejectedType);
        		typeDesignation.setConservedType(isConservedType);
        		typeDesignation.setCitation(citation);
        		typeDesignation.setCitationMicroReference(citationMicroReference);
        	}
        	
        	typeDesignationsList.remove(typeDesignation);
    		typeDesignationsList.add(typeDesignation);
        }
    }
    
	public class TemporaryNameTypeDesignation extends
			NameTypeDesignation {
				
		/**
		 * 
		 */
		private static final long serialVersionUID = 7309597024482795166L;

		public TemporaryNameTypeDesignation(TaxonNameBase<?, ?> typeName, ReferenceBase<?> citation, String citationMicroReference,
				String originalNameString, NameTypeDesignationStatus status, boolean isRejectedType, boolean isConservedType, boolean isNotDesignated) {
			super(typeName, citation, citationMicroReference, originalNameString, status,
					isRejectedType, isConservedType, isNotDesignated);
		}
	}
}
