/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.navigation.handler;

import java.util.UUID;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.IHandler;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.handlers.HandlerUtil;

import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.taxon.TaxonNode;
import eu.etaxonomy.taxeditor.navigation.NavigationUtil;
import eu.etaxonomy.taxeditor.operations.CreateTaxonNodeOperation;
import eu.etaxonomy.taxeditor.operations.IPostOperationEnabled;
import eu.etaxonomy.taxeditor.parser.ParserUtil;

/**
 * @author n.hoffmann
 * @created 24.03.2009
 * @version 1.0
 */
public class QuickCreateChildTaxonHandler extends AbstractHandler implements
		IHandler, IPostOperationEnabled {
	private static final Logger logger = Logger
			.getLogger(QuickCreateChildTaxonHandler.class);

	private UUID parentNodeUuid;

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.IHandler#execute(org.eclipse.core.commands.ExecutionEvent)
	 */
	public Object execute(ExecutionEvent event) throws ExecutionException {

		Shell shell = HandlerUtil.getActiveShell(event);
		
		CdmBase selection = NavigationUtil.getCurrentSelection(event);
		

		if((selection instanceof TaxonNode) && NavigationUtil.isDirty((TaxonNode) selection)){
			MessageDialog.openWarning(shell, "Unsaved Parent Taxon", "There are unsaved " +
					"changes in the parent taxon. Please save first.");
			return null;
		}
		
		parentNodeUuid = selection.getUuid();			
		
		InputDialog dialog = new InputDialog(shell, "Quick create child taxon", 
				"Enter child taxon's name to create a new taxon", null, null);
		if (dialog.open() == Window.OK) {
			
			//TaxonUtil.addChildTaxonBaseIsolated(parentTaxon.getUuid(), dialog.getValue());

			TaxonNameBase name = ParserUtil.parseReferencedName(dialog.getValue());
			
			CreateTaxonNodeOperation operation;
//			try {
//				operation = new CreateTaxonNodeOperation(event.getCommand().getName(), 
//						NavigationUtil.getUndoContext(), selection, name, this, null);
//				IStatus status = EditorUtil.executeOperation(operation);
//				if(status.getCode() == TaxeditorStorePlugin.ERROR_SAVING_ZERO_NAME){
//					MessageDialog.openWarning(shell, "No Name Specified", "An attempt was made to save a taxon with" +
//							" an empty name. Operation was cancelled");
//				}				
//			} catch (NotDefinedException e) {
//				logger.warn("Command name not set");
//			}
			
			logger.info("Taxon created");
			
		}
		
		return null;
	}

	public boolean postOperation(CdmBase objectAffectedByOperation) {
		NavigationUtil.selectInNavigator(objectAffectedByOperation, parentNodeUuid);
		return true;
	}
}
