/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.navigation.navigator;

import java.util.HashSet;
import java.util.Set;

import org.apache.log4j.Logger;

import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.taxon.Synonym;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.persistence.hibernate.CdmDataChangeEvent;
import eu.etaxonomy.cdm.persistence.hibernate.CdmDataChangeMap;
import eu.etaxonomy.cdm.persistence.hibernate.CdmDataChangeEvent.EventType;
import eu.etaxonomy.taxeditor.model.AbstractDataChangeBehaviour;
import eu.etaxonomy.taxeditor.model.IDataChangeBehavior;

/**
 * @author n.hoffmann
 * @created 01.04.2009
 * @version 1.0
 */
public class TaxonNavigatorDataChangeBehavior extends AbstractDataChangeBehaviour implements 
		IDataChangeBehavior {
	private static final Logger logger = Logger
			.getLogger(TaxonNavigatorDataChangeBehavior.class);

	private TaxonNavigator source;

	private Set<CdmBase> staleObjects;
	
	public TaxonNavigatorDataChangeBehavior(TaxonNavigator taxonNavigator) {
		source = taxonNavigator;
	}
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.store.model.IDataChangeBehavior#isRelevant(java.lang.Object, eu.etaxonomy.cdm.persistence.hibernate.CdmCrudEvent)
	 */
	public boolean isRelevant(CdmDataChangeMap events) {
		
		// TODO react only on insert/update/delete of taxon and synonym objects
		// and on update of name objects
		boolean relevant = false;
		staleObjects = new HashSet<CdmBase>();
		
		for(CdmDataChangeEvent event : events.getAllEvents()){
			EventType eventType = event.getEventType();
			// react on everything except load
			if(eventType == EventType.INSERT || eventType == EventType.UPDATE || eventType == EventType.DELETE){
				relevant = true;
				CdmBase entity = event.getEntity();
				if((entity instanceof TaxonNameBase)
						|| (entity instanceof Taxon)
						|| (entity instanceof Synonym)){
					staleObjects.add(entity);
				}
			}
		}
		
		return relevant;
		
		// @deprecated
		// react on everything except load
//		if(events.sizeByEventType(EventType.INSERT) > 0){
//			return true;
//		}else if(events.sizeByEventType(EventType.UPDATE) > 0){
//			return true;
//		}else if(events.sizeByEventType(EventType.DELETE) > 0){
//			return true;
//		}else{
//			return false;
//		}
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.store.model.IDataChangeBehavior#reactOnDataChange(java.lang.Object, eu.etaxonomy.cdm.persistence.hibernate.CdmCrudEvent)
	 */
	public void reactOnDataChange(CdmDataChangeMap events) {
		if(isRelevant(events)){

			// FIXME @see #1365		
			
			source.getConversationHolder().bind();
			
			// clear the session completely
			source.getConversationHolder().clear();
			
			// FIXME completely clearing the session is a brute force approach. 
			// It would be much more elegant to clear only those elements that have been changed.
			// I could not get that to work but we should consider workin on this because we might
			// run into serious performance issues, especially when it comes to large trees
			//
			// detach all changed objects from the session cache so that they have to be reloaded 
//			for (CdmBase staleObject : staleObjects){
//				source.getConversationHolder().evict(staleObject);
//			}
						
			source.refresh();			
			logger.info("DataStore update caused a refresh of the taxonomic tree");
		}
	}
}
