// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.io.wizard;

import org.apache.log4j.Logger;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.IWorkbench;

import eu.etaxonomy.cdm.database.ICdmDataSource;
import eu.etaxonomy.cdm.io.berlinModel.out.BerlinModelExportConfigurator;
import eu.etaxonomy.cdm.io.common.Source;
import eu.etaxonomy.taxeditor.datasource.wizard.CdmDataSourceSQLServerWizardPage;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author n.hoffmann
 * @created 23.06.2009
 * @version 1.0
 */
public class BerlinModelExportWizard extends AbstractExportWizard<BerlinModelExportConfigurator> {
	private static final Logger logger = Logger
			.getLogger(BerlinModelExportWizard.class);
	
	private CdmDataSourceSQLServerWizardPage pageDataSource;


	private BerlinModelExportConfigurator configurator;
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.IWorkbenchWizard#init(org.eclipse.ui.IWorkbench, org.eclipse.jface.viewers.IStructuredSelection)
	 */
	public void init(IWorkbench workbench, IStructuredSelection selection) {
		this.setWindowTitle("Berlin Model Export");
		configurator = BerlinModelExportConfigurator.NewInstance(null, null);
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.wizard.Wizard#addPages()
	 */
	@Override
	public void addPages() {
		super.addPages();
		
		pageDataSource = new CdmDataSourceSQLServerWizardPage(null);
		this.addPage(pageDataSource);
		

		
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.wizard.Wizard#performFinish()
	 */
	@Override
	public boolean performFinish() {
		
		ICdmDataSource dataSource = pageDataSource.getDataSource();
		
		// FIXME use new constructor Source(ICdmDataSource)
		Source destination = new Source(Source.SQL_SERVER_2005, 
								dataSource.getServer(),
								dataSource.getDatabase(),
								dataSource.getPort());
		destination.setUserAndPwd(dataSource.getUsername(), dataSource.getPassword());
		
		configurator.setDestination(destination);		
		
		CdmStore.getExportHandler().doExport(configurator);
		
		return true;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.io.AbstractExportWizard#getConfigurator()
	 */
	@Override
	public BerlinModelExportConfigurator getConfigurator() {
		return configurator;
	}

}
