// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.operations;

import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.runtime.IStatus;

import eu.etaxonomy.cdm.api.conversation.IConversationEnabled;
import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.model.taxon.ITreeNode;
import eu.etaxonomy.cdm.model.taxon.TaxonNode;

/**
 * @author n.hoffmann
 * @created 08.05.2009
 * @version 1.0
 */
public abstract class AbstractPersistentPostOperation extends AbstractPostOperation {
	private IConversationEnabled conversationEnabled;
	
	protected ITreeNode parentNode;
	
	/**
	 * 
	 * @param label
	 * @param undoContext
	 * @param postOperationEnabled
	 * @param conversationEnabled
	 */
	protected AbstractPersistentPostOperation(String label, IUndoContext undoContext,
			IPostOperationEnabled postOperationEnabled,
			IConversationEnabled conversationEnabled) {
		super(label, undoContext, postOperationEnabled);
		this.conversationEnabled = conversationEnabled;
	}
	
	/**
	 * 
	 * @param label
	 * @param undoContext
	 * @param taxonNode
	 * @param postOperationEnabled
	 * @param conversationEnabled
	 */
	public AbstractPersistentPostOperation(String label,
			IUndoContext undoContext, TaxonNode taxonNode,
			IPostOperationEnabled postOperationEnabled,
			IConversationEnabled conversationEnabled) {
		super(label, undoContext, taxonNode, postOperationEnabled);
		this.conversationEnabled = conversationEnabled;
	}
	
	/**
	 * 
	 * @param label
	 * @param undoContext
	 * @param taxonNode
	 * @param postOperationEnabled
	 * @param conversationEnabled
	 */
	public AbstractPersistentPostOperation(String label,
			IUndoContext undoContext, ITreeNode parentNode,
			IPostOperationEnabled postOperationEnabled,
			IConversationEnabled conversationEnabled) {
		super(label, undoContext, postOperationEnabled);
		this.parentNode = parentNode;
		this.conversationEnabled = conversationEnabled;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.operations.AbstractPostOperation#postExecute(eu.etaxonomy.cdm.model.common.CdmBase)
	 */
	protected IStatus postExecute(CdmBase objectAffectedByOperation) {	

		if(conversationEnabled != null){
			conversationEnabled.getConversationHolder().commit(true);
		}
		
		IStatus status = super.postExecute(objectAffectedByOperation);
		
		return status;
	}

}
