/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.operations;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;

import eu.etaxonomy.cdm.model.taxon.ITreeNode;
import eu.etaxonomy.cdm.model.taxon.Synonym;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonNode;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * Change the taxonomic parent of a given taxon.
 * 
 * @author n.hoffmann
 * @created 16.01.2009
 * @version 1.0
 */
public class ChangeSynonymToAcceptedTaxonOperation extends AbstractPostOperation {
	
	@SuppressWarnings("unused")
	private static final Logger logger = Logger.getLogger(ChangeSynonymToAcceptedTaxonOperation.class);

	private Taxon newTaxon;
	private Synonym synonym;
	private ITreeNode parentNode;

	private TaxonNode newNode;
	
	public ChangeSynonymToAcceptedTaxonOperation(String label, IUndoContext undoContext,
			Taxon taxon, ITreeNode parentNode, Synonym synonym, IPostOperationEnabled postOperationEnabled) {
		super(label, undoContext, taxon, postOperationEnabled);

		this.parentNode = parentNode;
		this.synonym = synonym; 
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#execute(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	@Override
	public IStatus execute(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		
		newTaxon = CdmStore.getTaxonService().changeSynonymToAcceptedTaxon(synonym, taxon);
		
//		synonym.setSec(null);
		
		// TODO let user choose parent
		newNode = parentNode.addChildTaxon(newTaxon, null, null, null);
				
		return postExecute(newNode);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#redo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	@Override
	public IStatus redo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		return execute(monitor, info);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#undo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	@Override
	public IStatus undo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		
		// TODO implement - biggest problem is that any window open for new taxon must be closed first
		logger.warn("Not yet implemented");
		
		return postExecute(taxon);
	}

}
