/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.operations;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;

import eu.etaxonomy.cdm.model.name.HomotypicalGroup;
import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.taxon.SynonymRelationship;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.parser.ParserUtil;

/**
 * @author n.hoffmann
 * @created 02.02.2009
 * @version 1.0
 */
public class CreateSynonymInExisitingHomotypicalGroupOperation extends
		AbstractPostOperation {

	private HomotypicalGroup group;
	private String newSynonymText;

	private SynonymRelationship synonymRelationship;
	
	public CreateSynonymInExisitingHomotypicalGroupOperation(String label,
			IUndoContext undoContext, Taxon taxon, HomotypicalGroup group, String newSynonymText, IPostOperationEnabled postOperationEnabled) {
		super(label, undoContext, taxon, postOperationEnabled);
		this.group = group;
		this.newSynonymText = newSynonymText;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#execute(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	@Override
	public IStatus execute(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		
		// Create synonym name using newSynonymText
		TaxonNameBase<?, ?> synonymName = 
				ParserUtil.parseReferencedName(newSynonymText);
		
		// Add name to given homotypic group
		group.addTypifiedName(synonymName);
		
		// Create a new synonym for the taxon
		// TODO add citations
		if(group.equals(taxon.getHomotypicGroup())){
			synonymRelationship = taxon.addHomotypicSynonymName(synonymName, null, null);
		}else{
			synonymRelationship = taxon.addHeterotypicSynonymName(synonymName);
		}
		
//		Synonym affectedSynonym = null;
//		
//		for(Synonym synonym : group.getSynonymsInGroup(taxon.getSec())){
//			if(synonym.getName() == synonymName){
//				affectedSynonym = synonym;
//				break;
//			}
//		}
		
		return postExecute(synonymRelationship.getSynonym());
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#redo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	@Override
	public IStatus redo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		return execute(monitor, info);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#undo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	@Override
	public IStatus undo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {

		taxon.removeSynonymRelation(synonymRelationship);
		
		return postExecute(null);
	}
}
