// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.operations;

import java.util.UUID;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.api.conversation.IConversationEnabled;
import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.taxon.ITreeNode;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonNode;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.store.internal.TaxeditorStorePlugin;

/**
 * @author n.hoffmann
 * @created 08.05.2009
 * @version 1.0
 */
public class CreateTaxonNodeOperation extends AbstractPersistentPostOperation {
	
	private static final Logger logger = Logger.getLogger(CreateTaxonNodeOperation.class);
	
	private Taxon newTaxon;
	
	private TaxonNode childTaxonNode;
	
	/**
	 * Add a name to a taxonomic tree
	 * 
	 * @param label
	 * @param undoContext
	 * @param parentNodeUuid
	 * @param name
	 * @param postOperationEnabled
	 */
	public CreateTaxonNodeOperation(String label, IUndoContext undoContext,
			ITreeNode parentNode, TaxonNameBase<?, ?> name, IPostOperationEnabled postOperationEnabled,
			IConversationEnabled conversationEnabled) {
		super(label, undoContext, parentNode, postOperationEnabled, conversationEnabled);
		
		newTaxon = Taxon.NewInstance(name, null);
	}
	
	/**
	 * Add a taxon to a taxonomic tree
	 * 
	 * @param label
	 * @param undoContext
	 * @param parentNodeUuid
	 * @param taxon
	 * @param postOperationEnabled
	 */
	public CreateTaxonNodeOperation(String label, IUndoContext undoContext,
			ITreeNode parentNode, Taxon taxon, IPostOperationEnabled postOperationEnabled,
			IConversationEnabled conversationEnabled) {
		super(label, undoContext, parentNode, postOperationEnabled, conversationEnabled);
		
		this.newTaxon = taxon;
	}


	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#execute(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	@Override
	public IStatus execute(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		
		newTaxon.setSec(parentNode.getReference());
		// add the taxon
		childTaxonNode = parentNode.addChildTaxon(newTaxon, parentNode.getReference(), parentNode.getMicroReference(), null);
		
		return postExecute(childTaxonNode);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#redo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	@Override
	public IStatus redo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		return execute(monitor, info);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#undo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	@Override
	public IStatus undo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		
		logger.warn("Not yet implemented.");
		return null;
	}
}
