// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.operations;

import java.util.UUID;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.widgets.Shell;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.api.conversation.IConversationEnabled;
import eu.etaxonomy.cdm.model.taxon.TaxonomicTree;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author n.hoffmann
 * @created 23.06.2009
 * @version 1.0
 */
public class DeleteTaxonomicTreeOperation extends AbstractPersistentPostOperation {
	
	private static final Logger logger = Logger
				.getLogger(DeleteTaxonomicTreeOperation.class);
	
	private TaxonomicTree taxonomicTree;


	/**
	 * @param label
	 * @param undoContext
	 * @param taxonNode
	 * @param postOperationEnabled
	 */
	public DeleteTaxonomicTreeOperation(String label, IUndoContext undoContext,
			TaxonomicTree taxonomicTree, IPostOperationEnabled postOperationEnabled,
			IConversationEnabled conversationEnabled) {
		super(label, undoContext, postOperationEnabled, conversationEnabled);

		this.taxonomicTree = taxonomicTree;		
	}	

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.IUndoableOperation#execute(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	public IStatus execute(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		
		// Prompt user for confirmation
		if(! MessageDialog.openConfirm((Shell) info.getAdapter(Shell.class), "Confirm Deletion", "Are you sure you want to delete the taxonomic view '" + taxonomicTree.getTitleCache() + "'?")){
			monitor.done();
			return Status.CANCEL_STATUS;
		}
		
		// If the taxonomic tree has child nodes, cancel operation
		// TODO add option to continue, and delete children
		if (taxonomicTree.getRootNodes().size() > 0) {
			MessageDialog.openInformation((Shell) info.getAdapter(Shell.class), "Cannot delete taxon", 
					"'" + taxonomicTree.getTitleCache() + "' has taxonomic children. " +
					"These must be manually deleted before their parent.");
			monitor.done();
			return Status.CANCEL_STATUS;
		}
		
		CdmStore.getTaxonTreeService().delete(taxonomicTree);
		
		return postExecute(null);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.IUndoableOperation#redo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	public IStatus redo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		// TODO Auto-generated method stub
		return null;
	}


	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.IUndoableOperation#undo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	public IStatus undo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		// TODO Auto-generated method stub
		return null;
	}
}
